from typing_extensions import Callable, Generic

from ..di import inject_deps
from ..exceptions import PluginIpcError
from ..mixin import AttrReprMixin, LocateMixin
from ..typ.base import AsyncCallable, T
from ..utils.common import RWContext


class AsyncShare(Generic[T], LocateMixin, AttrReprMixin):
    """异步共享对象"""

    def __init__(
        self,
        name: str,
        reflector: AsyncCallable[[], T] | None = None,
        callabck: AsyncCallable[[T], None] | None = None,
        static: bool = False,
    ) -> None:
        """初始化异步共享对象

        :param name: 异步共享对象的名称
        :param reflector: 获取共享值的异步可调用方法
        :param callabck: 修改共享值的异步可调用方法
        :param static: 是否使用静态模式
        """
        super().__init__()
        self.name = name
        self.__safe_ctx = RWContext()
        self.__reflect: AsyncCallable[[], T] | None = (
            inject_deps(reflector) if reflector is not None else None
        )
        self.__callback: AsyncCallable[[T], None] | None = (
            inject_deps(callabck, manual_arg=True) if callabck is not None else None
        )
        self.static = static

        if self.name.startswith("_"):
            raise PluginIpcError(f"共享对象 {self} 的名称不能以 _ 开头")
        if self.static and self.__callback is not None:
            raise PluginIpcError(
                f"{self} 作为静态的共享对象，不能绑定用于更新值的回调方法"
            )

    def __call__(self, func: AsyncCallable[[], T]) -> AsyncCallable[[], T]:
        """绑定获取共享值的异步方法的装饰器，如果未在初始化时绑定

        :param func: 被绑定的异步可调用方法
        :return: `func` 原值
        """
        if self.__reflect is not None:
            raise PluginIpcError("共享对象已经有获取值的反射方法，不能再次绑定")
        self.__reflect = inject_deps(func)
        return func

    def setter(self, func: AsyncCallable[[T], None]) -> AsyncCallable[[T], None]:
        """绑定修改共享值的异步方法的装饰器，如果未在初始化时绑定

        :param func: 被绑定的异步可调用方法
        :return: `func` 原值
        """
        if self.static:
            raise PluginIpcError(
                f"{self} 作为静态的共享对象，不能绑定用于更新值的回调方法"
            )
        if self.__callback is not None:
            raise PluginIpcError("共享对象已经有更新值的回调方法，不能再次绑定")
        self.__callback = inject_deps(func, manual_arg=True)
        return func

    async def get(self) -> T:
        """获取异步共享值

        :return: 异步共享值
        """
        if self.__reflect is None:
            raise PluginIpcError("共享对象获取值的反射方法未绑定")
        async with self.__safe_ctx.read():
            return await self.__reflect()

    async def set(self, val: T) -> None:
        """设置异步共享值

        :param val: 新的异步共享值
        """
        if self.__callback is None:
            raise PluginIpcError("共享对象更新值的回调方法未绑定")
        async with self.__safe_ctx.write():
            return await self.__callback(val)


class SyncShare(Generic[T], LocateMixin, AttrReprMixin):
    """同步共享对象"""

    def __init__(
        self,
        name: str,
        reflector: Callable[[], T] | None = None,
        callabck: Callable[[T], None] | None = None,
        static: bool = False,
    ) -> None:
        """初始化同步共享对象

        :param name: 同步共享对象的名称
        :param reflector: 获取共享值的可调用方法
        :param callabck: 修改共享值的可调用方法
        :param static: 是否使用静态模式
        """
        super().__init__()
        self.name = name
        self.__reflect = reflector
        self.__callback = callabck
        self.static = static

        if self.name.startswith("_"):
            raise PluginIpcError(f"共享对象 {self} 的名称不能以 _ 开头")
        if self.static and self.__callback is not None:
            raise PluginIpcError(
                f"{self} 作为静态的共享对象，不能绑定用于更新值的回调方法"
            )

    def __call__(self, func: Callable[[], T]) -> Callable[[], T]:
        """绑定获取共享值的方法的装饰器，如果未在初始化时绑定

        :param func: 被绑定的可调用方法
        :return: `func` 原值
        """
        if self.__reflect is not None:
            raise PluginIpcError("共享对象已经有获取值的反射方法，不能再次绑定")
        self.__reflect = func
        return func

    def setter(self, func: Callable[[T], None]) -> Callable[[T], None]:
        """绑定修改共享值的方法的装饰器，如果未在初始化时绑定

        :param func: 被绑定的可调用方法
        :return: `func` 原值
        """
        if self.static:
            raise PluginIpcError(
                f"{self} 作为静态的共享对象，不能绑定用于更新值的回调方法"
            )
        if self.__callback is not None:
            raise PluginIpcError("共享对象已经有更新值的回调方法，不能再次绑定")
        self.__callback = func
        return func

    def get(self) -> T:
        """获取共享值

        :return: 共享值
        """
        if self.__reflect is None:
            raise PluginIpcError("共享对象未绑定获取值的反射方法")
        return self.__reflect()

    def set(self, val: T) -> None:
        """设置共享值

        :param val: 新的共享值
        """
        if self.__callback is None:
            raise PluginIpcError("共享对象未绑定更新值的回调方法")
        self.__callback(val)


class IPCManager:
    def __init__(self) -> None:
        self._shares: dict[str, dict[str, AsyncShare | SyncShare]] = {}

    def add(self, plugin: str, obj: AsyncShare | SyncShare) -> None:
        objs = self._shares.setdefault(plugin, {})
        if objs.get(obj.name) is not None:
            raise PluginIpcError(f"插件 {plugin} 中已存在名为 {obj.name} 的共享对象")
        objs[obj.name] = obj

    def add_func(self, plugin: str, func: Callable) -> None:
        self.add(plugin, SyncShare(func.__name__, lambda: func, None, True))

    def get(self, plugin: str, id: str) -> AsyncShare | SyncShare:
        if (objs := self._shares.get(plugin)) is None:
            raise PluginIpcError(f"插件 {plugin} 未加载，或其不提供共享功能")
        if (obj := objs.get(id)) is None:
            raise PluginIpcError(f"无法获取不存在的共享对象：标识 {id} 不存在")
        return obj



from typing_extensions import Any, ParamSpec, TypeVar

T = TypeVar("T", default=Any)
V = TypeVar("V", default=Any)
P = ParamSpec("P", default=Any)


PROTOCOL_NAME = "OneBot"
PROTOCOL_VERSION = "11"
PROTOCOL_SUPPORT_AUTHOR = "Meloland"
PROTOCOL_IDENTIFIER = f"{PROTOCOL_NAME}-v{PROTOCOL_VERSION}@{PROTOCOL_SUPPORT_AUTHOR}"

ACTION_TYPE_KEY_NAME = "action_type"



import asyncio as aio

import pytest as pt
import pytest_asyncio as ptaio
from pytest import fixture
from pytest_asyncio import fixture as aiofixture

# Auto use "package" loop_scope (not pytest fixture scope) for all async test functions
pytestmark = pt.mark.asyncio(loop_scope="package")



from __future__ import annotations

from pydantic import BaseModel
from typing_extensions import Any, Literal, Sequence, cast

from melobot.adapter import Event as RootEvent
from melobot.adapter import TextEvent as RootTextEvent
from melobot.adapter import content

from ..const import PROTOCOL_IDENTIFIER
from .segment import Segment, TextSegment, segs_to_contents


class Event(RootEvent):
    class Model(BaseModel):
        time: int
        self_id: int
        post_type: Literal["message", "notice", "request", "meta_event"] | str

    def __init__(self, **event_data: Any) -> None:
        self._model = self.Model(**event_data)
        #: 时间戳
        self.time: int

        super().__init__(PROTOCOL_IDENTIFIER, self._model.time)
        #: 机器人自己的 qq 号
        self.self_id: int = self._model.self_id
        #: 事件类型
        self.post_type: Literal["message", "notice", "request", "meta_event"] | str = (
            self._model.post_type
        )
        #: 事件原始数据
        self.raw: dict[str, Any] = event_data

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(post_type={self.post_type})"

    @classmethod
    def resolve(cls, event_data: dict[str, Any]) -> Event:
        cls_map: dict[str, type[Event]] = {
            "message": MessageEvent,
            "notice": NoticeEvent,
            "request": RequestEvent,
            "meta_event": MetaEvent,
        }
        if (etype := event_data.get("post_type")) in cls_map:
            return cls_map[etype].resolve(event_data)
        return cls(**event_data)

    def is_message(self) -> bool:
        return self.post_type == "message"

    def is_notice(self) -> bool:
        return self.post_type == "notice"

    def is_request(self) -> bool:
        return self.post_type == "request"

    def is_meta(self) -> bool:
        return self.post_type == "meta_event"


class MessageEvent(RootTextEvent, Event):
    class Model(Event.Model):
        post_type: Literal["message"]
        message_type: Literal["private", "group"] | str
        sub_type: (
            Literal[
                "friend", "group", "other", "normal", "anonymous", "notice", "group_self"
            ]
            | str
        )
        message_id: int
        user_id: int
        raw_message: str
        font: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: MessageEvent.Model
        #: 消息内容（消息段表示）
        self.message: list[Segment]
        #: 消息发送者
        self.sender: _MessageSender | _GroupMessageSender
        #: 消息内容（通用内容表示）
        self.contents: Sequence[content.Content] = []
        #: 消息内容（cq 字符串表示）
        self.raw_message: str

        self.scope: tuple[int, ...]

        data = event_data
        if isinstance(data["message"], str):
            self.message = Segment.__resolve_cq__(data["raw_message"])
        else:
            self.message = [
                Segment.resolve(dic["type"], dic["data"]) for dic in data["message"]
            ]
        self.contents = segs_to_contents(self.message)

        #: 消息类型
        self.message_type: Literal["private", "group"] | str = self._model.message_type
        #: 消息子类型
        self.sub_type: (
            Literal[
                "friend", "group", "other", "normal", "anonymous", "notice", "group_self"
            ]
            | str
        ) = self._model.sub_type
        #: 消息 id
        self.message_id: int = self._model.message_id
        #: 消息发送者 ID
        self.user_id: int = self._model.user_id
        self.raw_message = self._model.raw_message
        #: 消息字体
        self.font: int = self._model.font

        #: 消息内容
        self.text = "".join(
            seg.data["text"] for seg in self.message if isinstance(seg, TextSegment)
        )
        #: 消息内容行
        self.textlines = "\n".join(
            seg.data["text"] for seg in self.message if isinstance(seg, TextSegment)
        ).split("\n")

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}(text={self.text!r},"
            f" user_id={self.user_id}), sub_type={self.sub_type})"
        )

    @classmethod
    def resolve(cls, event_data: dict[str, Any]) -> MessageEvent:
        cls_map: dict[str, type[MessageEvent]] = {
            "private": PrivateMessageEvent,
            "group": GroupMessageEvent,
        }
        if (mtype := event_data.get("message_type")) in cls_map:
            return cls_map[mtype](**event_data)
        return cls(**event_data)

    def get_segments(self, type: type[Segment] | str) -> list[Segment]:
        if isinstance(type, str):
            return [seg for seg in self.message if seg.type == type]
        return [seg for seg in self.message if isinstance(seg, type)]

    def get_datas(self, type: type[Segment] | str, name: str) -> list[Any]:
        segs = self.get_segments(type)
        return [seg.data.get(name, None) for seg in segs]

    def is_private(self) -> bool:
        """是否为私聊消息（注意群临时会话属于该类别）"""
        return self.message_type == "private" or self.is_private_temp()

    def is_friend(self) -> bool:
        """是否为好友消息"""
        return self.is_private() and self.sub_type == "friend"

    def is_group_temp(self) -> bool:
        """是否为群临时会话（属于私聊的一种）"""
        return self.is_private() and self.sub_type == "group"

    def is_private_other(self) -> bool:
        """是否为其他私聊类型消息"""
        return self.is_private() and self.sub_type == "other"

    def is_private_temp(self) -> bool:
        """是否为临时会话（属于私聊的一种）"""
        return "temp_source" in self.raw

    def is_group(self) -> bool:
        """是否为群消息（正常群消息、群匿名消息、群自身消息、群系统消息属于该类型）"""
        return self.message_type == "group"

    def is_group_normal(self) -> bool:
        """是否为正常群消息"""
        return self.is_group() and self.sub_type == "normal"

    def is_group_self(self) -> bool:
        """是否为群自身消息（即 bot 自己群中发的消息）"""
        return self.is_group() and self.sub_type == "group_self"

    def is_group_anonymous(self) -> bool:
        """是否为匿名群消息"""
        return self.is_group() and self.sub_type == "anonymous"

    def is_group_notice(self) -> bool:
        """是否为群中的\"系统消息\" """
        return self.is_group() and self.sub_type == "notice"


class _MessageSender:

    class Model(BaseModel):
        user_id: int | None = None
        nickname: str | None = None
        sex: Literal["male", "female", "unknown"] | None = None
        age: int | None = None

    def __init__(self, **sender_data: Any) -> None:
        self._model = self.Model(**sender_data)

        #: 发送者 ID
        self.user_id: int | None = self._model.user_id
        #: 发送者昵称
        self.nickname: str | None = self._model.nickname
        #: 发送者性别
        self.sex: Literal["male", "female", "unknown"] | None = self._model.sex
        #: 发送者年龄
        self.age: int | None = self._model.age

    def is_group_owner(self) -> bool:
        """判断是否为群主，若不是或不是群类型消息，返回 False"""
        if self.__class__ is not _GroupMessageSender:
            return False

        _self = cast(_GroupMessageSender, self)
        return (
            _self.role is not None and _self.role == "owner"  # pylint: disable=no-member
        )

    def is_group_admin(self) -> bool:
        """判断是否为群管理（包含群主），若不是或不是群类型消息，返回 False"""
        if self.__class__ is not _GroupMessageSender:
            return False

        _self = cast(_GroupMessageSender, self)
        return _self.role is not None and _self.role in (  # pylint: disable=no-member
            "owner",
            "admin",
        )

    def is_group_member_only(self) -> bool:
        """判断是否只是群员（注意只是群员，不包括群主、管理和匿名），若不是或不是群类型消息，返回 False"""
        if self.__class__ is not _GroupMessageSender:
            return False

        _self = cast(_GroupMessageSender, self)
        return (
            _self.role is not None and _self.role == "member"  # pylint: disable=no-member
        )


class PrivateMessageEvent(MessageEvent):

    class Model(MessageEvent.Model):
        message_type: Literal["private"]
        sub_type: Literal["friend", "group", "other"] | str

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        #: 消息发送者
        self.sender: _MessageSender
        self.sender = _MessageSender(**event_data["sender"])
        self.scope = (-1, self.user_id)

        self._model: PrivateMessageEvent.Model
        #: 消息类型
        self.message_type: Literal["private"]
        # 消息子类型
        self.sub_type: Literal["friend", "group", "other"]


class _MessageAnonymous:

    class Model(BaseModel):
        id: int
        name: str
        flag: str

    def __init__(self, **anonymous_data: Any) -> None:
        self._model = self.Model(**anonymous_data)

        #: 匿名信息的 id
        self.id: int = self._model.id
        #: 匿名信息的 name
        self.name: str = self._model.name
        #: 匿名信息的 flag
        self.flag: str = self._model.flag


class _GroupMessageSender(_MessageSender):

    class Model(_MessageSender.Model):
        card: str | None = None
        area: str | None = None
        level: str | None = None
        role: Literal["owner", "admin", "member"] | None = None
        title: str | None = None

    def __init__(self, **sender_data: Any) -> None:
        self._model: _GroupMessageSender.Model
        super().__init__(**sender_data)

        #: 发送者头衔
        self.card: str | None = self._model.card
        #: 发送者地区
        self.area: str | None = self._model.area
        #: 发送者等级
        self.level: str | None = self._model.level
        #: 发送者角色
        self.role: Literal["owner", "admin", "member"] | None = self._model.role
        #: 发送者群昵称
        self.title: str | None = self._model.title


class GroupMessageEvent(MessageEvent):

    class Model(MessageEvent.Model):
        message_type: Literal["group"]
        sub_type: Literal["normal", "anonymous", "notice", "group_self"] | str
        group_id: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        #: 消息发送者
        self.sender: _GroupMessageSender
        self.sender = _GroupMessageSender(**event_data["sender"])
        #: 消息匿名信息段
        self.anonymous: _MessageAnonymous | None = (
            _MessageAnonymous(**event_data["anonymous"])
            if event_data["anonymous"]
            else None
        )
        #: 消息来源群号
        self.group_id: int = self._model.group_id
        self.scope = (self.group_id, self.user_id)

        self._model: GroupMessageEvent.Model
        #: 消息类型
        self.message_type: Literal["group"]
        #: 消息子类型
        self.sub_type: Literal["normal", "anonymous", "notice", "group_self"]

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}(text={self.text!r},"
            f" user_id={self.user_id}, group_id={self.group_id},"
            f" sub_type={self.sub_type})"
        )


class MetaEvent(Event):

    class Model(Event.Model):
        post_type: Literal["meta_event"]
        meta_event_type: Literal["lifecycle", "heartbeat"] | str

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: MetaEvent.Model
        #: 元事件类型
        self.meta_event_type: Literal["lifecycle", "heartbeat"] | str = (
            self._model.meta_event_type
        )

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}(meta_type={self.meta_event_type}"
            + f", sub_type={self.sub_type})"
            if hasattr(self, "sub_type")
            else ")"
        )

    @classmethod
    def resolve(cls, event_data: dict[str, Any]) -> MetaEvent:
        cls_map: dict[str, type[MetaEvent]] = {
            "lifecycle": LifeCycleMetaEvent,
            "heartbeat": HeartBeatMetaEvent,
        }
        if (mtype := event_data.get("meta_event_type")) in cls_map:
            return cls_map[mtype](**event_data)
        return cls(**event_data)

    def is_lifecycle(self) -> bool:
        return self.meta_event_type == "lifecycle"

    def is_heartbeat(self) -> bool:
        return self.meta_event_type == "heartbeat"


class LifeCycleMetaEvent(MetaEvent):

    class Model(MetaEvent.Model):
        meta_event_type: Literal["lifecycle"]
        sub_type: Literal["enable", "disable", "connect"] | str

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: LifeCycleMetaEvent.Model
        #: 元事件类型
        self.meta_event_type: Literal["lifecycle"]
        #: 元事件子类型
        self.sub_type: Literal["enable", "disable", "connect"] | str = (
            self._model.sub_type
        )

    def is_enable(self) -> bool:
        return self.sub_type == "enable"

    def is_disable(self) -> bool:
        return self.sub_type == "disable"

    def is_connect(self) -> bool:
        return self.sub_type == "connect"


class _MetaHeartBeatStatus:

    class Model(BaseModel):
        online: bool | None
        good: bool

    def __init__(self, **status_data: Any) -> None:
        self._model = self.Model(**status_data)

        #: 心跳元事件在线状态
        self.online: bool | None = self._model.online
        #: 心跳元事件健康状态
        self.good: bool = self._model.good
        #: 心跳元事件原始内容
        self.raw: dict[str, Any] = status_data


class HeartBeatMetaEvent(MetaEvent):

    class Model(MetaEvent.Model):
        meta_event_type: Literal["heartbeat"]
        interval: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: HeartBeatMetaEvent.Model

        #: 元事件类型
        self.meta_event_type: Literal["heartbeat"]
        #: 心跳元事件状态
        self.status: _MetaHeartBeatStatus = _MetaHeartBeatStatus(**event_data["status"])
        #: 心跳间隔
        self.interval: int = self._model.interval


class NoticeEvent(Event):

    class Model(Event.Model):
        post_type: Literal["notice"]
        notice_type: (
            Literal[
                "group_upload",
                "group_admin",
                "group_decrease",
                "group_increase",
                "group_ban",
                "friend_add",
                "group_recall",
                "friend_recall",
                "notify",
            ]
            | str
        )

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: NoticeEvent.Model
        #: 通知事件类型
        self.notice_type: (
            Literal[
                "group_upload",
                "group_admin",
                "group_decrease",
                "group_increase",
                "group_ban",
                "friend_add",
                "group_recall",
                "friend_recall",
                "notify",
            ]
            | str
        ) = self._model.notice_type

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}(notice_type={self.notice_type}"
            + f", sub_type={self.sub_type})"
            if hasattr(self, "sub_type")
            else ")"
        )

    @classmethod
    def resolve(cls, event_data: dict[str, Any]) -> NoticeEvent:
        cls_map: dict[str, type[NoticeEvent]] = {
            "group_upload": GroupUploadNoticeEvent,
            "group_admin": GroupAdminNoticeEvent,
            "group_decrease": GroupDecreaseNoticeEvent,
            "group_increase": GroupIncreaseNoticeEvent,
            "group_ban": GroupBanNoticeEvent,
            "friend_add": FriendAddNoticeEvent,
            "group_recall": GroupRecallNoticeEvent,
            "friend_recall": FriendRecallNoticeEvent,
        }
        ntype = event_data.get("notice_type")
        if ntype in cls_map:
            return cls_map[ntype](**event_data)
        if ntype == "notify":
            return NotifyNoticeEvent.resolve(event_data)
        return cls(**event_data)

    def is_group_upload(self) -> bool:
        """是否是群文件上传通知事件"""
        return self.notice_type == "group_upload"

    def is_group_admin(self) -> bool:
        """是否是群管理员变动通知事件"""
        return self.notice_type == "group_admin"

    def is_group_decrease(self) -> bool:
        """是否是群成员减少通知事件"""
        return self.notice_type == "group_decrease"

    def is_group_increase(self) -> bool:
        """是否是群成员增加通知事件"""
        return self.notice_type == "group_increase"

    def is_group_ban(self) -> bool:
        """是否是群禁言通知事件"""
        return self.notice_type == "group_ban"

    def is_friend_add(self) -> bool:
        """是否是好友添加事件"""
        return self.notice_type == "friend_add"

    def is_group_recall(self) -> bool:
        """是否是群消息撤回事件"""
        return self.notice_type == "group_recall"

    def is_friend_recall(self) -> bool:
        """是否是私聊消息撤回事件"""
        return self.notice_type == "friend_recall"

    def is_notify(self) -> bool:
        """是否是 notify 类型通知事件"""
        return self.notice_type == "notify"


class _GroupUploadFile:

    class Model(BaseModel):
        id: str
        name: str
        size: int
        busid: int

    def __init__(self, **file_data: Any) -> None:
        self._model = self.Model(**file_data)

        #: 群文件 id
        self.id: str = self._model.id
        #: 群文件名称
        self.name: str = self._model.name
        #: 群文件大小
        self.size: int = self._model.size
        #: 群文件 busid
        self.busid: int = self._model.busid


class GroupUploadNoticeEvent(NoticeEvent):

    class Model(NoticeEvent.Model):
        notice_type: Literal["group_upload"]
        group_id: int
        user_id: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: GroupUploadNoticeEvent.Model

        #: 通知事件类型
        self.notice_type: Literal["group_upload"]
        #: 群号
        self.group_id: int = self._model.group_id
        #: 事件主体人 qq 号
        self.user_id: int = self._model.user_id
        #: 文件描述对象
        self.file: _GroupUploadFile = _GroupUploadFile(**event_data["file"])


class GroupAdminNoticeEvent(NoticeEvent):

    class Model(NoticeEvent.Model):
        notice_type: Literal["group_admin"]
        sub_type: Literal["set", "unset"]
        group_id: int
        user_id: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: GroupAdminNoticeEvent.Model

        #: 通知事件类型
        self.notice_type: Literal["group_admin"]
        #: 子类型
        self.sub_type: Literal["set", "unset"] = self._model.sub_type
        #: 群号
        self.group_id: int = self._model.group_id
        #: 事件主体人 qq 号
        self.user_id: int = self._model.user_id

    def is_set(self) -> bool:
        return self.sub_type == "set"

    def is_unset(self) -> bool:
        return self.sub_type == "unset"


class GroupDecreaseNoticeEvent(NoticeEvent):

    class Model(NoticeEvent.Model):
        notice_type: Literal["group_decrease"]
        sub_type: Literal["kick", "leave", "kick_me"]
        group_id: int
        operator_id: int
        user_id: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: GroupDecreaseNoticeEvent.Model

        #: 通知事件类型
        self.notice_type: Literal["group_decrease"]
        #: 子类型
        self.sub_type: Literal["kick", "leave", "kick_me"] = self._model.sub_type
        #: 群号
        self.group_id: int = self._model.group_id
        #: 操作者 qq 号
        self.operator_id: int = self._model.operator_id
        #: 事件主体人 qq 号
        self.user_id: int = self._model.user_id

    def is_kick(self) -> bool:
        return self.sub_type == "kick"

    def is_kick_me(self) -> bool:
        return self.sub_type == "kick_me"

    def is_leave(self) -> bool:
        return self.sub_type == "leave"


class GroupIncreaseNoticeEvent(NoticeEvent):

    class Model(NoticeEvent.Model):
        notice_type: Literal["group_increase"]
        sub_type: Literal["invite", "approve"]
        group_id: int
        operator_id: int
        user_id: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: GroupIncreaseNoticeEvent.Model

        #: 通知事件类型
        self.notice_type: Literal["group_increase"]
        #: 子类型
        self.sub_type: Literal["invite", "approve"] = self._model.sub_type
        #: 群号
        self.group_id: int = self._model.group_id
        #: 操作者 qq 号
        self.operator_id: int = self._model.operator_id
        #: 事件主体人 qq 号
        self.user_id: int = self._model.user_id

    def is_invite(self) -> bool:
        return self.sub_type == "invite"

    def is_approve(self) -> bool:
        return self.sub_type == "approve"


class GroupBanNoticeEvent(NoticeEvent):

    class Model(NoticeEvent.Model):
        notice_type: Literal["group_ban"]
        sub_type: Literal["ban", "lift_ban"]
        group_id: int
        operator_id: int
        user_id: int
        duration: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: GroupBanNoticeEvent.Model

        #: 通知事件类型
        self.notice_type: Literal["group_ban"]
        #: 子类型
        self.sub_type: Literal["ban", "lift_ban"] = self._model.sub_type
        #: 群号
        self.group_id: int = self._model.group_id
        #: 操作者 qq 号
        self.operator_id: int = self._model.operator_id
        #: 事件主体人 qq 号
        self.user_id: int = self._model.user_id
        #: 禁言间隔
        self.duration: int = self._model.duration

    def is_ban(self) -> bool:
        return self.sub_type == "ban"

    def is_lift_ban(self) -> bool:
        return self.sub_type == "lift_ban"


class FriendAddNoticeEvent(NoticeEvent):

    class Model(NoticeEvent.Model):
        notice_type: Literal["friend_add"]
        user_id: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: FriendAddNoticeEvent.Model

        #: 通知事件类型
        self.notice_type: Literal["friend_add"]
        #: 事件主体人 qq 号
        self.user_id: int = self._model.user_id


class GroupRecallNoticeEvent(NoticeEvent):

    class Model(NoticeEvent.Model):
        notice_type: Literal["group_recall"]
        group_id: int
        user_id: int
        operator_id: int
        message_id: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: GroupRecallNoticeEvent.Model

        #: 通知事件类型
        self.notice_type: Literal["group_recall"]
        #: 群号
        self.group_id: int = self._model.group_id
        #: 事件主体人 qq 号
        self.user_id: int = self._model.user_id
        #: 操作者 qq 号
        self.operator_id: int = self._model.operator_id
        #: 被撤回的消息 id
        self.message_id: int = self._model.message_id


class FriendRecallNoticeEvent(NoticeEvent):

    class Model(NoticeEvent.Model):
        notice_type: Literal["friend_recall"]
        user_id: int
        message_id: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: FriendRecallNoticeEvent.Model

        #: 通知事件类型
        self.notice_type: Literal["friend_recall"]
        #: 事件主体人 qq 号
        self.user_id: int = self._model.user_id
        #: 被撤回的消息 id
        self.message_id: int = self._model.message_id


class NotifyNoticeEvent(NoticeEvent):

    class Model(NoticeEvent.Model):
        notice_type: Literal["notify"]
        sub_type: Literal["poke", "lucky_king", "honor"] | str

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: NotifyNoticeEvent.Model

        #: 通知事件类型
        self.notice_type: Literal["notify"]
        #: 子类型
        self.sub_type: Literal["poke", "lucky_king", "honor"] | str = self._model.sub_type

    @classmethod
    def resolve(cls, event_data: dict[str, Any]) -> NotifyNoticeEvent:
        cls_map: dict[str, type[NotifyNoticeEvent]] = {
            "poke": PokeNotifyEvent,
            "lucky_king": LuckyKingNotifyEvent,
            "honor": HonorNotifyEvent,
        }
        if (stype := event_data.get("sub_type")) in cls_map:
            return cls_map[stype](**event_data)
        return cls(**event_data)

    def is_poke(self) -> bool:
        return self.sub_type == "poke"

    def is_lucky_king(self) -> bool:
        return self.sub_type == "lucky_king"

    def is_honor(self) -> bool:
        return self.sub_type == "honor"


class PokeNotifyEvent(NotifyNoticeEvent):

    class Model(NotifyNoticeEvent.Model):
        sub_type: Literal["poke"]
        group_id: int | None = None
        user_id: int
        target_id: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: PokeNotifyEvent.Model
        #: 子类型
        self.sub_type: Literal["poke"]
        #: 群号
        self.group_id: int | None = self._model.group_id
        #: 事件发起者 qq 号
        self.user_id: int = self._model.user_id
        #: 事件作用者 qq 号
        self.target_id: int = self._model.target_id


class LuckyKingNotifyEvent(NotifyNoticeEvent):

    class Model(NotifyNoticeEvent.Model):
        sub_type: Literal["lucky_king"]
        group_id: int
        user_id: int
        target_id: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: LuckyKingNotifyEvent.Model

        #: 子类型
        self.sub_type: Literal["lucky_king"]
        #: 群号
        self.group_id: int = self._model.group_id
        #: 红包发送者 qq 号
        self.user_id: int = self._model.user_id
        #: 运气王 qq 号
        self.target_id: int = self._model.target_id


class HonorNotifyEvent(NotifyNoticeEvent):

    class Model(NotifyNoticeEvent.Model):
        sub_type: Literal["honor"]
        group_id: int
        honor_type: Literal["talkative", "performer", "emotion"]
        user_id: int

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: HonorNotifyEvent.Model

        #: 子类型
        self.sub_type: Literal["honor"]
        #: 群号
        self.group_id: int = self._model.group_id
        #: 群荣誉类型
        self.honor_type: Literal["talkative", "performer", "emotion"] = (
            self._model.honor_type
        )
        #: 事件主体人 qq 号
        self.user_id: int = self._model.user_id

    def is_talkative(self) -> bool:
        return self.honor_type == "talkative"

    def is_performer(self) -> bool:
        return self.honor_type == "performer"

    def is_emotion(self) -> bool:
        return self.honor_type == "emotion"


class RequestEvent(Event):

    class Model(Event.Model):
        post_type: Literal["request"]
        request_type: Literal["friend", "group"] | str

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: RequestEvent.Model
        #: 请求事件类型
        self.request_type: Literal["friend", "group"] | str = self._model.request_type

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}(request_type={self.request_type}"
            + f", sub_type={self.sub_type})"
            if hasattr(self, "sub_type")
            else ")"
        )

    @classmethod
    def resolve(cls, event_data: dict[str, Any]) -> RequestEvent:
        cls_map: dict[str, type[RequestEvent]] = {
            "friend": FriendRequestEvent,
            "group": GroupRequestEvent,
        }
        if (rtype := event_data.get("request_type")) in cls_map:
            return cls_map[rtype](**event_data)
        return cls(**event_data)

    def is_friend(self) -> bool:
        return self.request_type == "friend"

    def is_group(self) -> bool:
        return self.request_type == "group"


class FriendRequestEvent(RequestEvent):

    class Model(RequestEvent.Model):
        request_type: Literal["friend"]
        user_id: int
        comment: str
        flag: str

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: FriendRequestEvent.Model

        #: 请求事件类型
        self.request_type: Literal["friend"]
        #: 请求人 qq 号
        self.user_id: int = self._model.user_id
        #: 加好友备注
        self.comment: str = self._model.comment
        #: 请求 flag，在调用处理请求的 API 时需要传入
        self.flag: str = self._model.flag


class GroupRequestEvent(RequestEvent):

    class Model(RequestEvent.Model):
        request_type: Literal["group"]
        sub_type: Literal["add", "invite"]
        group_id: int
        user_id: int
        comment: str
        flag: str

    def __init__(self, **event_data: Any) -> None:
        super().__init__(**event_data)

        self._model: GroupRequestEvent.Model
        #: 请求事件类型
        self.request_type: Literal["group"]
        #: 子类型
        self.sub_type: str = self._model.sub_type
        #: 群号
        self.group_id: int = self._model.group_id
        #: 请求人 qq 号
        self.user_id: int = self._model.user_id
        #: 加群备注
        self.comment: str = self._model.comment
        #: 请求 flag，在调用处理请求的 API 时需要传入
        self.flag: str = self._model.flag

    def is_add(self) -> bool:
        return self.sub_type == "add"

    def is_invite(self) -> bool:
        return self.sub_type == "invite"



import argparse
import sys

from melobot._cli import dev, pinit, run

parser = argparse.ArgumentParser(prog="mb-cli", description="melobot 命令行工具")
sub_parsers = parser.add_subparsers()

pinit_parser = sub_parsers.add_parser("pinit", help="运行 melobot 插件自动初始化")
pinit_parser.add_argument("files", nargs="*", help="插件根目录路径")
pinit_parser.add_argument(
    "--depth", "-d", type=int, default=1, help="插件初始化的向上引用深度"
)
pinit_parser.set_defaults(_cmd_handler=pinit.main)

run_parser = sub_parsers.add_parser(
    "run", help="以子进程模式运行 melobot bot 主程序，且支持重启"
)
run_parser.add_argument("entry_file", help="bot 程序入口 .py 文件路径")
run_parser.set_defaults(_cmd_handler=run.main)

dev_parser = sub_parsers.add_parser(
    "dev", help="以开发模式运行 melobot bot 主程序，支持重启与自动重载"
)
dev_parser.add_argument("entry_file", help="bot 程序入口 .py 文件路径")
dev_parser.add_argument("--watch", nargs="*", default=["."], help="需要监测的文件路径")
dev_parser.set_defaults(_cmd_handler=dev.main)

args = parser.parse_args()
if len(sys.argv) > 1:
    args._cmd_handler(args)
else:
    print("无命令参数，mb-cli 已结束运行，使用 -h 命令参数获取帮助信息")



from os import listdir as _VAR1
from pathlib import Path as _VAR2
from typing import Any as _VAR3

from melobot import get_bot as _VAR4

_VAR5 = _VAR2(__file__).parent
_VAR6 = set(fname.split(".")[0] for fname in _VAR1(_VAR5))
_VAR7 = _VAR5.parts[-1]


def __getattr__(name: str) -> _VAR3:
    if name in _VAR6 or name.startswith("_"):
        raise AttributeError
    obj = _VAR4().get_share(_VAR7, name)
    if obj.static:
        return obj.get()
    return obj



from typing import cast

from melobot.adapter.content import TextContent
from melobot.protocols.onebot.v11.adapter import event
from tests.base import *


@fixture
def base_head():
    return {"time": 1725292489, "self_id": 123456}


@fixture
def msg_head(base_head):
    return base_head | {"post_type": "message"}


@fixture
def private_sender():
    return {
        "age": 0,
        "nickname": "这是一个群昵称",
        "sex": "unknown",
        "user_id": 1574260633,
    }


@fixture
def group_sender(private_sender):
    return private_sender | {
        "area": "",
        "card": "",
        "level": "",
        "role": "member",
        "title": "",
    }


async def test_msg(msg_head, private_sender, group_sender) -> None:
    e1 = event.Event.resolve(
        msg_head
        | {
            "message_type": "group",
            "sub_type": "normal",
            "sender": group_sender,
            "message_id": -1234567890,
            "font": 0,
            "message": "123456",
            "user_id": 1574260633,
            "anonymous": None,
            "group_id": 535705163,
            "raw_message": "123456",
        }
    )
    assert cast(TextContent, e1.contents[0]).text == "123456"

    e2 = event.Event.resolve(
        msg_head
        | {
            "message_type": "group",
            "sub_type": "anonymous",
            "sender": group_sender,
            "message_id": -1234567890,
            "font": 0,
            "message": "",
            "user_id": 1574260633,
            "anonymous": {"id": 123, "name": "test", "flag": "adjfajklj12j32l"},
            "group_id": 535705163,
            "raw_message": "",
        }
    )
    e3 = event.Event.resolve(
        msg_head
        | {
            "message_type": "group",
            "sub_type": "notice",
            "sender": group_sender,
            "message_id": -1234567890,
            "font": 0,
            "message": "",
            "user_id": 1574260633,
            "anonymous": None,
            "group_id": 535705163,
            "raw_message": "",
        }
    )
    e4 = event.Event.resolve(
        msg_head
        | {
            "message_type": "group",
            "sub_type": "group_self",
            "sender": group_sender,
            "message_id": -1234567890,
            "font": 0,
            "message": "",
            "user_id": 1574260633,
            "anonymous": None,
            "group_id": 535705163,
            "raw_message": "",
        }
    )
    e5 = event.Event.resolve(
        msg_head
        | {
            "message_type": "private",
            "sub_type": "friend",
            "sender": private_sender,
            "message_id": -1234567890,
            "font": 0,
            "message": "",
            "user_id": 1574260633,
            "raw_message": "",
        }
    )
    e6 = event.Event.resolve(
        msg_head
        | {
            "message_type": "private",
            "sub_type": "group",
            "sender": private_sender,
            "message_id": -1234567890,
            "font": 0,
            "message": "",
            "user_id": 1574260633,
            "raw_message": "",
        }
    )
    e7 = event.Event.resolve(
        msg_head
        | {
            "message_type": "private",
            "sub_type": "other",
            "sender": private_sender,
            "message_id": -1234567890,
            "font": 0,
            "message": "",
            "user_id": 1574260633,
            "raw_message": "",
        }
    )
    assert isinstance(e1, event.GroupMessageEvent)
    assert isinstance(e2, event.GroupMessageEvent)
    assert isinstance(e3, event.GroupMessageEvent)
    assert isinstance(e4, event.GroupMessageEvent)
    assert isinstance(e5, event.PrivateMessageEvent)
    assert isinstance(e6, event.PrivateMessageEvent)
    assert isinstance(e7, event.PrivateMessageEvent)


@fixture
def meta_head(base_head):
    return base_head | {"post_type": "meta_event"}


@fixture
def lifecycle_head(meta_head):
    return meta_head | {"meta_event_type": "lifecycle"}


async def test_meta(meta_head, lifecycle_head) -> None:
    e1 = event.Event.resolve(lifecycle_head | {"sub_type": "enable"})
    e2 = event.Event.resolve(lifecycle_head | {"sub_type": "disable"})
    e3 = event.Event.resolve(lifecycle_head | {"sub_type": "connect"})
    e4 = event.Event.resolve(
        meta_head
        | {
            "meta_event_type": "heartbeat",
            "status": {"online": True, "good": True},
            "interval": 5000,
        }
    )
    assert isinstance(e1, event.LifeCycleMetaEvent)
    assert isinstance(e2, event.LifeCycleMetaEvent)
    assert isinstance(e3, event.LifeCycleMetaEvent)
    assert isinstance(e4, event.HeartBeatMetaEvent)


@fixture
def notice_head(base_head):
    return base_head | {"post_type": "notice"}


@fixture
def notify_head(notice_head):
    return notice_head | {"notice_type": "notify"}


async def test_notice(notice_head, notify_head) -> None:
    e = event.Event.resolve(
        notice_head
        | {
            "notice_type": "group_upload",
            "group_id": 123456,
            "user_id": 123456,
            "file": {"id": "123", "name": "test.jpg", "size": 123456, "busid": -123},
        }
    )
    assert isinstance(e, event.GroupUploadNoticeEvent)

    e1 = event.Event.resolve(
        notice_head
        | {
            "notice_type": "group_admin",
            "sub_type": "set",
            "group_id": 123456,
            "user_id": 123456,
        }
    )
    e2 = event.Event.resolve(
        notice_head
        | {
            "notice_type": "group_admin",
            "sub_type": "unset",
            "group_id": 123456,
            "user_id": 123456,
        }
    )
    assert isinstance(e1, event.GroupAdminNoticeEvent)
    assert isinstance(e2, event.GroupAdminNoticeEvent)

    e1 = event.Event.resolve(
        notice_head
        | {
            "notice_type": "group_decrease",
            "sub_type": "kick",
            "group_id": 123456,
            "user_id": 123456,
            "operator_id": 123456,
        }
    )
    e2 = event.Event.resolve(
        notice_head
        | {
            "notice_type": "group_decrease",
            "sub_type": "kick_me",
            "group_id": 123456,
            "user_id": 123456,
            "operator_id": 123456,
        }
    )
    e3 = event.Event.resolve(
        notice_head
        | {
            "notice_type": "group_decrease",
            "sub_type": "leave",
            "group_id": 123456,
            "user_id": 123456,
            "operator_id": 123456,
        }
    )
    assert isinstance(e1, event.GroupDecreaseNoticeEvent)
    assert isinstance(e2, event.GroupDecreaseNoticeEvent)
    assert isinstance(e3, event.GroupDecreaseNoticeEvent)

    e1 = event.Event.resolve(
        notice_head
        | {
            "notice_type": "group_increase",
            "sub_type": "invite",
            "group_id": 123456,
            "user_id": 123456,
            "operator_id": 123456,
        }
    )
    e2 = event.Event.resolve(
        notice_head
        | {
            "notice_type": "group_increase",
            "sub_type": "approve",
            "group_id": 123456,
            "user_id": 123456,
            "operator_id": 123456,
        }
    )
    assert isinstance(e1, event.GroupIncreaseNoticeEvent)
    assert isinstance(e2, event.GroupIncreaseNoticeEvent)

    e1 = event.Event.resolve(
        notice_head
        | {
            "notice_type": "group_ban",
            "sub_type": "ban",
            "group_id": 123456,
            "user_id": 123456,
            "operator_id": 123456,
            "duration": 3600,
        }
    )
    e2 = event.Event.resolve(
        notice_head
        | {
            "notice_type": "group_ban",
            "sub_type": "lift_ban",
            "group_id": 123456,
            "user_id": 123456,
            "operator_id": 123456,
            "duration": 3600,
        }
    )
    assert isinstance(e1, event.GroupBanNoticeEvent)
    assert isinstance(e2, event.GroupBanNoticeEvent)

    e1 = event.Event.resolve(
        notice_head | {"notice_type": "friend_add", "user_id": 123456}
    )
    assert isinstance(e1, event.FriendAddNoticeEvent)

    e1 = event.Event.resolve(
        notice_head
        | {
            "notice_type": "group_recall",
            "group_id": 123456,
            "user_id": 123456,
            "operator_id": 123456,
            "message_id": -12345678,
        }
    )
    assert isinstance(e1, event.GroupRecallNoticeEvent)

    e1 = event.Event.resolve(
        notice_head
        | {"notice_type": "friend_recall", "user_id": 123456, "message_id": -12345678}
    )
    assert isinstance(e1, event.FriendRecallNoticeEvent)

    e1 = event.Event.resolve(
        notify_head
        | {
            "sub_type": "poke",
            "group_id": 123456,
            "user_id": 123456,
            "target_id": 123456,
        }
    )
    e2 = event.Event.resolve(
        notify_head
        | {
            "sub_type": "lucky_king",
            "group_id": 123456,
            "user_id": 123456,
            "target_id": 123456,
        }
    )
    e3 = event.Event.resolve(
        notify_head
        | {
            "sub_type": "honor",
            "group_id": 123456,
            "user_id": 123456,
            "honor_type": "talkative",
        }
    )
    e4 = event.Event.resolve(
        notify_head
        | {
            "sub_type": "honor",
            "group_id": 123456,
            "user_id": 123456,
            "honor_type": "performer",
        }
    )
    e5 = event.Event.resolve(
        notify_head
        | {
            "sub_type": "honor",
            "group_id": 123456,
            "user_id": 123456,
            "honor_type": "emotion",
        }
    )
    assert isinstance(e1, event.PokeNotifyEvent)
    assert isinstance(e2, event.LuckyKingNotifyEvent)
    assert isinstance(e3, event.HonorNotifyEvent)
    assert isinstance(e4, event.HonorNotifyEvent)
    assert isinstance(e5, event.HonorNotifyEvent)


@fixture
def request_head(base_head):
    return base_head | {"post_type": "request"}


async def test_request(request_head) -> None:
    e1 = event.Event.resolve(
        request_head
        | {
            "request_type": "friend",
            "user_id": 123456,
            "comment": "I want to add you as my friend",
            "flag": "asdfja78a97a07f",
        }
    )
    e2 = event.Event.resolve(
        request_head
        | {
            "request_type": "group",
            "sub_type": "add",
            "group_id": 123456,
            "user_id": 123456,
            "comment": "I want to join your group",
            "flag": "asdfja78a97a07f",
        }
    )
    e3 = event.Event.resolve(
        request_head
        | {
            "request_type": "group",
            "sub_type": "invite",
            "group_id": 123456,
            "user_id": 123456,
            "comment": "I want to join your group",
            "flag": "asdfja78a97a07f",
        }
    )
    assert isinstance(e1, event.FriendRequestEvent)
    assert isinstance(e2, event.GroupRequestEvent)



import os
import subprocess
import sys
from argparse import Namespace
from pathlib import Path

from melobot.bot.base import CLI_RUNTIME, LAST_EXIT_SIGNAL, BotExitSignal


def main(args: Namespace) -> None:
    entry_path = Path(args.entry_file)
    if not entry_path.is_absolute():
        entry = str(entry_path.resolve())
    else:
        entry = str(entry_path)

    if not entry.endswith(".py"):
        entry += ".py"
    if not Path(entry).exists():
        print(f"不存在的入口文件：{str(entry)}")
        sys.exit(1)

    cmd = [sys.executable, entry]
    cwd = str(Path.cwd().resolve())
    os.environ[CLI_RUNTIME] = "1"

    try:
        while True:
            proc = subprocess.run(
                cmd,
                env=os.environ,
                cwd=cwd,
                stdin=sys.stdin,
                stdout=sys.stdout,
                stderr=sys.stderr,
                check=False,
            )
            if LAST_EXIT_SIGNAL in os.environ:
                os.environ.pop(LAST_EXIT_SIGNAL)

            if proc.returncode == BotExitSignal.NORMAL_STOP.value:
                break
            if proc.returncode == BotExitSignal.RESTART.value:
                print("\n>>> [melobot module] 正在重启 bot 主程序\n")
                os.environ[LAST_EXIT_SIGNAL] = str(BotExitSignal.RESTART.value)
                continue
            if proc.returncode == BotExitSignal.ERROR.value:
                break
            print("\n>>> [melobot module] bot 主程序退出时返回了无法处理的返回值\n")
            break

    except KeyboardInterrupt:
        pass



from asyncio import Queue

from melobot.ctx import LoggerCtx
from melobot.log import Logger
from melobot.utils.parse import CmdArgFormatter as Fmtter
from melobot.utils.parse import CmdParserFactory
from tests.base import *

_OUT_BUF = Queue()


async def test_parser():
    pf1 = CmdParserFactory(".", ["#", "$"])
    pf2 = CmdParserFactory([".", "*"], " ")

    p1 = pf1.get("test")
    assert (await p1.parse("as;dfja;j;f;ajf;")) is None
    assert (await p1.parse("\n\t .test 123")) is None
    assert (await p1.parse("\n\t .test123#123")) is None
    assert (await p1.parse("\n\t .test\t\n")).vals == []
    assert (await p1.parse("\n\t .test#456\n\t\r\n")).vals == ["456"]

    p2 = pf1.get(["test", "echo"])
    assert (await p2.parse(".test#asdjf;a#")).vals == ["asdjf;a"]
    assert (await p2.parse(".echo#")).vals == []

    with LoggerCtx().unfold(Logger()):
        p3 = pf1.get(
            ["test", "echo"],
            [
                Fmtter(
                    convert=lambda s: int(s),
                    validate=lambda i: 0 <= i <= 10,
                    src_desc="测试命令的重复次数",
                    src_expect="0-10 的数字",
                    default=5,
                    default_replace_flag="/",
                    convert_fail=lambda _: _OUT_BUF.put(True),
                    validate_fail=lambda _: _OUT_BUF.put(True),
                    arg_lack=lambda _: _OUT_BUF.put(True),
                ),
                Fmtter(
                    convert=lambda s: float(s),
                    validate=lambda f: 0.0 <= f <= 10.0,
                    src_desc="测试命令的间隔时间",
                    src_expect="0.0-10.0 的数字",
                    arg_lack=lambda _: _OUT_BUF.put(True),
                ),
            ],
        )
        await p3.parse(".test#123")
        _OUT_BUF.get_nowait()
        await p3.parse(".echo#/$")
        _OUT_BUF.get_nowait()
        await p3.parse(".echo#abc")
        _OUT_BUF.get_nowait()
        assert (await p3.parse(".test$10#2")).vals == [10, 2.0]
        assert (await p3.parse(".echo#/$8")).vals == [5, 8.0]

    p4 = pf2.get(["test", "echo"])
    assert (await p4.parse("\t\f\n\rasdfa\n\rfa .test 123   asjf;\n\rlaja\r\n")).vals == [
        "123",
        "asjf;\n\rlaja",
    ]
    assert (
        await p4.parse("\t\f\n\rasdfa\n\rfa *echo 456 asdaf;   asjf;\n\rlaja\r\n")
    ).vals == ["456", "asdaf;", "asjf;\n\rlaja"]



import logging
import sys

import better_exceptions
from typing_extensions import Any

better_exceptions.SUPPORTS_COLOR = True
better_exceptions.color.SUPPORTS_COLOR = True
better_exceptions.formatter.SUPPORTS_COLOR = True
# 修复在 windows powershell 显示错误的 bug
better_exceptions.encoding.ENCODING = sys.stdout.encoding
better_exceptions.formatter.ENCODING = sys.stdout.encoding
# 直接 hook，而不是让它使用环境变量触发
sys.excepthook = better_exceptions.excepthook
# 取消它的猴子补丁
logging._loggerClass = (  # type:ignore[attr-defined]
    logging.Logger
)


class BotException(Exception):
    """bot 异常基类"""

    def __init__(self, *args: object):
        super().__init__(self, args)
        if not len(args):
            self.err = ""
        elif len(args) == 1:
            self.err = str(args[0])
        else:
            self.err = str(args)
        self.pretty_err = (
            f"[{self.__class__.__module__}.{self.__class__.__qualname__}] {self.err}"
        )

    def __str__(self) -> str:
        return self.err


class UtilError(BotException):
    """melobot.utils 异常"""


class UtilValidateError(UtilError):
    """:py:mod:`melobot.utils` 函数参数验证异常"""


class BotError(BotException):
    """melobot bot 异常"""


class SourceError(BotException):
    """melobot 源异常"""


class PluginError(BotException):
    """melobot 插件异常"""


class PluginAutoGenError(PluginError):
    """melobot 插件自动生成异常"""


class PluginLoadError(PluginError):
    """melobot 插件加载异常"""


class PluginIpcError(PluginError):
    """melobot 插件间通信异常"""


class SessionError(BotException):
    """melobot 会话异常"""


class SessionStateFailed(SessionError):
    def __init__(self, cur_state: str, meth: str) -> None:
        self.cur_state = cur_state
        super().__init__(f"当前会话状态 {cur_state} 不支持的操作：{meth}")


class SessionRuleLacked(SessionError): ...


class FlowError(BotException):
    """melobot 处理流异常"""


class LogError(BotException):
    """melobot 日志器异常"""


class HookError(BotException):
    """melobot 生命周期组件异常"""


class AdapterError(BotException):
    """melobot 适配器异常"""


class DependError(BotException):
    """melobot 依赖注入异常"""


class DependInitError(DependError):
    """melobot 依赖注入项初始化失败"""


class DependBindError(DependError):
    """melobot 依赖注入项值绑定失败"""


class DynamicImpError(BotException, ImportError):
    """melobot 动态导入组件异常"""

    def __init__(
        self, *args: Any, name: str | None = None, path: str | None = None
    ) -> None:
        BotException.__init__(self, *args)
        ImportError.__init__(self, *args, name=name, path=path)



from ..ctx import SessionCtx as _SessionCtx
from .base import Session, SessionStore, enter_session, suspend
from .option import CompareInfo, DefaultRule, Rule


def get_session() -> Session:
    """获取当前上下文中的会话

    :return: 会话
    """
    return _SessionCtx().get()


def get_session_store() -> SessionStore:
    """获取当前上下文中的会话存储

    :return: 会话存储
    """
    return _SessionCtx().get().store


def get_rule() -> Rule:
    """获取当前上下文中的会话规则

    :return: 会话规则
    """
    rule = _SessionCtx().get().rule
    assert rule is not None, "预期之外的会话规则为空"
    return rule



from typing_extensions import Any, Callable, Sequence, cast

from melobot.adapter.model import Event as RootEvent
from melobot.handle import FlowDecorator
from melobot.handle import on_event as on_root_event
from melobot.handle import on_text
from melobot.session import Rule
from melobot.utils.check import Checker, checker_join
from melobot.utils.common import DeprecatedLoader as _DeprecatedLoader
from melobot.utils.match import Matcher
from melobot.utils.parse import Parser

from .adapter.event import Event, MessageEvent, MetaEvent, NoticeEvent, RequestEvent
from .utils import check


def on_event(
    checker: Checker | None | Callable[[Event], bool] = None,
    matcher: Matcher | None = None,
    parser: Parser | None = None,
    priority: int = 0,
    block: bool = False,
    temp: bool = False,
    decos: Sequence[Callable[[Callable], Callable]] | None = None,
    rule: Rule[Event] | None = None,
) -> FlowDecorator:
    return on_root_event(
        checker=checker_join(lambda e: isinstance(e, Event), checker),
        matcher=matcher,
        parser=parser,
        priority=priority,
        block=block,
        temp=temp,
        decos=decos,
        rule=cast(Rule[RootEvent] | None, rule),
    )


def on_message(
    checker: Checker | None | Callable[[MessageEvent], bool] = None,
    matcher: Matcher | None = None,
    parser: Parser | None = None,
    priority: int = 0,
    block: bool = False,
    temp: bool = False,
    decos: Sequence[Callable[[Callable], Callable]] | None = None,
    legacy_session: bool = False,
) -> FlowDecorator:
    return on_text(
        checker_join(lambda e: isinstance(e, MessageEvent), checker),
        matcher,
        parser,
        priority,
        block,
        temp,
        decos,
        legacy_session,
    )


def on_at_qq(
    qid: int | None = None,
    checker: Checker | None | Callable[[MessageEvent], bool] = None,
    matcher: Matcher | None = None,
    parser: Parser | None = None,
    priority: int = 0,
    block: bool = False,
    temp: bool = False,
    decos: Sequence[Callable[[Callable], Callable]] | None = None,
    legacy_session: bool = False,
) -> FlowDecorator:
    return on_message(
        checker_join(check.AtMsgChecker(qid if qid else "all"), checker),  # type: ignore[arg-type]
        matcher,
        parser,
        priority,
        block,
        temp,
        decos,
        legacy_session,
    )


def on_request(
    checker: Checker | None | Callable[[RequestEvent], bool] = None,
    matcher: Matcher | None = None,
    parser: Parser | None = None,
    priority: int = 0,
    block: bool = False,
    temp: bool = False,
    decos: Sequence[Callable[[Callable], Callable]] | None = None,
    rule: Rule[Event] | None = None,
) -> FlowDecorator:
    return on_event(
        checker_join(lambda e: isinstance(e, RequestEvent), checker),  # type: ignore[arg-type]
        matcher,
        parser,
        priority,
        block,
        temp,
        decos,
        rule,
    )


def on_notice(
    checker: Checker | None | Callable[[NoticeEvent], bool] = None,
    matcher: Matcher | None = None,
    parser: Parser | None = None,
    priority: int = 0,
    block: bool = False,
    temp: bool = False,
    decos: Sequence[Callable[[Callable], Callable]] | None = None,
    rule: Rule[Event] | None = None,
) -> FlowDecorator:
    return on_event(
        checker_join(lambda e: isinstance(e, NoticeEvent), checker),  # type: ignore[arg-type]
        matcher,
        parser,
        priority,
        block,
        temp,
        decos,
        rule,
    )


def on_meta(
    checker: Checker | None | Callable[[MetaEvent], bool] = None,
    matcher: Matcher | None = None,
    parser: Parser | None = None,
    priority: int = 0,
    block: bool = False,
    temp: bool = False,
    decos: Sequence[Callable[[Callable], Callable]] | None = None,
    rule: Rule[Event] | None = None,
) -> FlowDecorator:
    return on_event(
        checker_join(lambda e: isinstance(e, MetaEvent), checker),  # type: ignore[arg-type]
        matcher,
        parser,
        priority,
        block,
        temp,
        decos,
        rule,
    )


_LOADER = _DeprecatedLoader(
    __name__,
    {
        "GetParseArgs": ("melobot.handle", "GetParseArgs", "3.1.1"),
        "on_command": ("melobot.handle", "on_command", "3.1.1"),
        "on_start_match": ("melobot.handle", "on_start_match", "3.1.1"),
        "on_contain_match": ("melobot.handle", "on_contain_match", "3.1.1"),
        "on_full_match": ("melobot.handle", "on_full_match", "3.1.1"),
        "on_end_match": ("melobot.handle", "on_end_match", "3.1.1"),
        "on_regex_match": ("melobot.handle", "on_regex_match", "3.1.1"),
    },
)


def __getattr__(name: str) -> Any:
    return _LOADER.get(name)



from __future__ import annotations

from abc import abstractmethod
from asyncio import Lock
from collections import deque
from dataclasses import dataclass
from functools import partial, wraps
from inspect import Parameter, isawaitable, signature, unwrap
from sys import version_info
from types import BuiltinFunctionType, FunctionType, LambdaType

from typing_extensions import (
    TYPE_CHECKING,
    Annotated,
    Any,
    Callable,
    Generic,
    Sequence,
    cast,
    get_args,
    get_origin,
)

from .ctx import BotCtx, EventOrigin, FlowCtx, LoggerCtx, ParseArgsCtx, SessionCtx
from .exceptions import DependBindError, DependInitError
from .typ._enum import VoidType
from .typ.base import AsyncCallable, P, SyncOrAsyncCallable, T, is_subhint, is_type
from .typ.cls import BetterABC
from .utils.base import to_async
from .utils.common import get_obj_name

if TYPE_CHECKING:
    from .adapter.base import Adapter


class DependNotMatched(BaseException):
    def __init__(
        self, msg: str, func_name: str, arg_name: str, real_type: type, hint: Any
    ) -> None:
        super().__init__(msg)
        self.func_name = func_name
        self.arg_name = arg_name
        self.real_type = real_type
        self.hint = hint


class Depends(Generic[T]):
    def __init__(
        self,
        dep: SyncOrAsyncCallable[[], T] | Depends[T],
        sub_getter: SyncOrAsyncCallable[[T], T] | None = None,
        cache: bool = False,
        recursive: bool = True,
    ) -> None:
        """初始化一个依赖项

        :param dep: 依赖来源（可调用对象，异步可调用对象，或依赖项）
        :param sub_getter: 子获取器（可调用对象，异步可调用对象或空），在获得依赖之后，于其上继续获取
        :param cache: 是否启用缓存
        :param recursive: 是否启用递归满足（默认启用，如果当前依赖来源中存在依赖项，会被递归满足；关闭可节约性能）
        """
        super().__init__()
        self.ref: Depends[T] | None
        self.getter: AsyncCallable[[], T] | None

        if isinstance(dep, Depends):
            self.ref = dep
            self.getter = None
        else:
            self.ref = None
            if recursive:
                self.getter = inject_deps(dep)
            else:
                self.getter = to_async(dep)

        if sub_getter is None:
            self.sub_getter = None
        elif recursive:
            self.sub_getter = inject_deps(sub_getter)
        else:
            self.sub_getter = to_async(sub_getter)

        self._lock = Lock() if cache else None
        self._cached: Any = VoidType.VOID

    def __repr__(self) -> str:
        getter_str = f"getter={self.getter}" if self.getter is not None else ""
        ref_str = f"ref={self.ref}" if self.ref is not None else ""
        return f"{self.__class__.__name__}({ref_str if ref_str != '' else getter_str})"

    async def _get(self, dep_scope: dict[Depends, Any]) -> T:
        val: T | VoidType

        if self.getter is not None:
            val = await self.getter()
        else:
            ref = cast(Depends[T], self.ref)
            val = dep_scope.get(ref, VoidType.VOID)
            if val is VoidType.VOID:
                val = await ref.fulfill(dep_scope)

        if self.sub_getter is not None:
            val = await self.sub_getter(val)
        return val

    async def fulfill(self, dep_scope: dict[Depends, Any]) -> T:
        if self._lock is None:
            val = await self._get(dep_scope)
        elif self._cached is not VoidType.VOID:
            val = self._cached
        else:
            async with self._lock:
                if self._cached is VoidType.VOID:
                    self._cached = await self._get(dep_scope)
                val = self._cached

        dep_scope[self] = val
        return val


class AutoDepends(Depends):
    def __init__(self, func: Callable, name: str, hint: Any) -> None:
        self.hint = hint
        self.func = func
        self.func_name = get_obj_name(func, otype="callable")
        self.arg_name = name

        if get_origin(hint) is Annotated:
            args = get_args(hint)
            if not len(args):
                raise DependInitError(
                    "可依赖注入的函数若使用 Annotated 注解，必须附加元数据"
                )
            self.metadatas = args
        else:
            self.metadatas = ()

        self.orig_getter: SyncOrAsyncCallable[[], Any] | None = None

        if is_subhint(hint, FlowCtx().get_event_type()):
            self.orig_getter = FlowCtx().get_event

        elif is_subhint(hint, BotCtx().get_type()):
            self.orig_getter = BotCtx().get

        elif is_subhint(hint, _get_adapter_type()):
            self.orig_getter = cast(Callable[[], Any], partial(_adapter_get, self, hint))

        elif is_subhint(hint, LoggerCtx().get_type()):
            self.orig_getter = LoggerCtx().get

        elif is_subhint(hint, FlowCtx().get_store_type()):
            self.orig_getter = FlowCtx().get_store

        elif is_subhint(hint, SessionCtx().get_session_type()):
            self.orig_getter = SessionCtx().get

        elif is_subhint(hint, SessionCtx().get_store_type()):
            self.orig_getter = SessionCtx().get_store

        elif is_subhint(hint, SessionCtx().get_rule_type()):
            self.orig_getter = SessionCtx().get_rule

        elif is_subhint(hint, ParseArgsCtx().get_args_type()):
            self.orig_getter = ParseArgsCtx().get

        for data in self.metadatas:
            if isinstance(data, CustomLogger):
                self.orig_getter = cast(
                    Callable[[], Any], partial(_custom_logger_get, hint, data)
                )
                break

        if self.orig_getter is None:
            raise DependInitError(
                f"函数 {self.func_name} 的参数 {name} 提供的类型注解"
                f" {hint} 无法用于注入任何依赖，请检查是否有误"
            )

        for data in self.metadatas:
            if isinstance(data, Reflect):
                self.orig_getter = cast(
                    Callable[[], Any], partial(Reflection, self.orig_getter)
                )
                break

        super().__init__(self.orig_getter, sub_getter=None, cache=False, recursive=False)

    def _unmatch_exc(self, real_type: Any) -> DependNotMatched:
        return DependNotMatched(
            f"函数 {self.func_name} 的参数 {self.arg_name} " f"与注解 {self.hint} 不匹配",
            self.func_name,
            self.arg_name,
            real_type,
            self.hint,
        )

    def _match_check(self, val: Any) -> None:
        for data in self.metadatas:
            if isinstance(data, Exclude):
                if any(isinstance(val, t) for t in data.types):
                    raise self._unmatch_exc(type(val))

        for data in self.metadatas:
            if isinstance(data, CustomLogger):
                return

        if not is_type(val, self.hint):
            raise self._unmatch_exc(type(val))

    async def fulfill(self, dep_scope: dict[Depends, Any]) -> Any:
        val = await super().fulfill(dep_scope)

        if isinstance(val, Reflection):
            inner_val = val.__origin__
            if isawaitable(inner_val):
                raise AttributeError(
                    f"异步依赖项不能通过 {Reflect.__name__} 创建反射依赖"
                )

            self._match_check(inner_val)
            return val

        self._match_check(val)
        return val


def _get_adapter_type() -> type["Adapter"]:
    from .adapter.base import Adapter

    return Adapter


def _adapter_get(deps: AutoDepends, hint: Any) -> "Adapter":
    flow_ctx = FlowCtx()
    try:
        event = flow_ctx.get_event()
        return EventOrigin.get_origin(event).adapter
    except flow_ctx.lookup_exc_cls:
        adapter = BotCtx().get().get_adapter(hint)
        if adapter is None:
            raise deps._unmatch_exc(VoidType) from None
        return adapter


def _custom_logger_get(hint: Any, data: CustomLogger) -> Any:
    val = LoggerCtx().get()
    if not is_type(val, hint):
        val = data.getter()
    return val


@dataclass
class Exclude:
    """数据类。`types` 指定的类别会在依赖注入时被排除

    .. code:: python

        # 假设有继承关系 A <- B, A <- C, A <- D
        # 表示 A 中不包括 B 和 C 类别的所有子类型，当然，还是会兼容 A 类型本身
        NewTypeHint = Annotated[A, Exclude(types=[B, C])]
    """

    types: Sequence[type]


@dataclass
class CustomLogger:
    """数据类。`getter` 参数会用于指定类别日志器不存在时的获取方法

    .. code:: python

        # 如果 bot 设置的 logger 是 MyLogger 类型，则成功依赖注入
        # 否则使用 getter 获取一个日志器
        NewLoggerHint = Annotated[MyLogger, CustomLogger(getter=MyLogger)]
    """

    getter: Callable[[], Any]


@dataclass
class Reflect:
    """数据类。指定不直接获取当前依赖项，而是获取对应的一个反射代理

    这适用于希望依赖会随着上下文改变，而动态变化的情况。例如动态引用会话流程中的事件对象

    .. code:: python

        # 注入一个依赖时进一步包装为反射依赖
        event_proxy = Annotated[Event, Reflect()]
        # 就像使用 event 一样使用 event_proxy
        event_proxy.attr_xxx
        event_proxy.method_xxx()

        # 不过 event_proxy 不是完美的代理
        # 因此 isinstance 类似的操作，使用 __origin__ 获取原始对象
        isinstance(event_proxy.__origin__, SomeEventType)
        # 或者是作为运行逻辑未知的函数的参数
        dont_know_what_this_do(event_proxy.__origin__)
    """


class Reflection:
    def __init__(self, getter: Callable[[], Any]) -> None:
        super().__setattr__("__obj_getter__", getter)

    @property
    def __origin__(self) -> Any:
        return self.__obj_getter__()

    def __getattr__(self, name: str) -> Any:
        getter = self.__obj_getter__
        if name == "__obj_getter__":
            return getter
        if name.startswith("_"):
            raise AttributeError(f"在反射对象上，不允许访问名称以 _ 开头的属性：{name}")

        return getattr(getter(), name)

    def __setattr__(self, name: str, value: Any) -> Any:
        getter = self.__obj_getter__
        if name == "__obj_getter__":
            return getter
        if name.startswith("_"):
            raise AttributeError(f"在反射对象上，不允许修改名称以 _ 开头的属性：{name}")

        return setattr(getter(), name, value)


def _init_auto_deps(func: Callable[P, T], allow_manual_arg: bool) -> None:
    try:
        sign = signature(func)
    except ValueError as e:
        tip = "no signature found for builtin"
        if str(e).startswith(tip) and version_info <= (3, 10):
            raise DependInitError(
                f"内建函数 {func} 在 python <= 3.10 的版本中，无法进行依赖注入"
            ) from None
        raise

    empty = Parameter.empty
    origin_f = unwrap(func, stop=lambda f: hasattr(f, "__signature__"))
    ds = deque(origin_f.__defaults__) if origin_f.__defaults__ is not None else deque()
    kwds = origin_f.__kwdefaults__ if origin_f.__kwdefaults__ is not None else {}
    nargs: list[Any] = []

    for name, param in sign.parameters.items():
        if param.kind in (Parameter.VAR_POSITIONAL, Parameter.VAR_KEYWORD):
            continue

        if param.default is not empty:
            if name in kwds:
                pass
            else:
                ds.popleft()
                nargs.append(param.default)
            continue

        if param.annotation is empty:
            continue

        try:
            dep = AutoDepends(func, name, param.annotation)
        except DependInitError:
            if allow_manual_arg:
                continue
            raise

        if dep is None:
            continue

        if param.kind is Parameter.KEYWORD_ONLY:
            kwds[name] = dep
        else:
            nargs.append(dep)

    origin_f.__defaults__ = tuple(nargs) if len(nargs) else None
    origin_f.__kwdefaults__ = kwds if len(kwds) else None  # type: ignore[assignment]


def _get_bound_args(
    func: Callable, /, *args: Any, **kwargs: Any
) -> tuple[list[Any], dict[str, Any]]:
    sign = signature(func)

    try:
        bind = sign.bind(*args, **kwargs)
    except TypeError as e:
        fname = get_obj_name(func, otype="callable")
        raise DependBindError(
            f"依赖注入匹配失败。匹配函数 {fname} 的参数时发生错误：{e}。"
            "这可能是因为传参个数不匹配，或提供了错误的类型注解"
        ) from None

    bind.apply_defaults()
    return list(bind.args), bind.kwargs


class DependsHook(Depends[T], BetterABC):
    """依赖钩子

    包装一个依赖项，依赖满足后内部的 hook 将会执行
    """

    def __init__(
        self,
        dep: SyncOrAsyncCallable[[], T],
        cache: bool = False,
        recursive: bool = False,
    ) -> None:
        super().__init__(dep, cache=cache, recursive=recursive)

    @abstractmethod
    async def deps_callback(self, val: T) -> None:
        """所有依赖钩子子类必须实现该抽象方法

        :param val: 依赖项被满足后的值
        """
        raise NotImplementedError

    async def fulfill(self, dep_scope: dict[Depends, Any]) -> T:
        val = await super().fulfill(dep_scope)
        await self.deps_callback(val)
        return val


def inject_deps(
    injectee: SyncOrAsyncCallable[..., T], manual_arg: bool = False
) -> AsyncCallable[..., T]:
    """依赖注入标记装饰器，标记当前对象需要被依赖注入

    可以标记的对象类别有：
    同步函数，异步函数，匿名函数，同步生成器函数，异步生成器函数，实例方法、类方法、静态方法

    :param injectee: 需要被注入的对象
    :param manual_arg: 当前对象标记需要依赖注入后，是否还可以给某些参数手动传参
    :return: 异步可调用对象，但保留原始参数和返回值签名
    """

    @wraps(injectee)
    async def inject_deps_wrapped(*args: Any, **kwargs: Any) -> T:
        _args, _kwargs = _get_bound_args(injectee, *args, **kwargs)
        dep_scope: dict[Depends, Any] = {}

        for idx, _ in enumerate(_args):
            elem = _args[idx]
            if isinstance(elem, Depends):
                _args[idx] = await elem.fulfill(dep_scope)

        for idx, k in enumerate(_kwargs.keys()):
            elem = _kwargs[k]
            if isinstance(elem, Depends):
                _kwargs[k] = await elem.fulfill(dep_scope)

        ret = injectee(*_args, **_kwargs)  # type: ignore[arg-type]
        if isawaitable(ret):
            return await ret
        return ret

    if isinstance(injectee, (FunctionType, BuiltinFunctionType)):
        _init_auto_deps(injectee, manual_arg)
        return inject_deps_wrapped
    if isinstance(injectee, LambdaType):
        return inject_deps_wrapped

    raise DependInitError(
        f"{injectee} 对象不属于以下类别中的任何一种："
        "{同步函数，异步函数，匿名函数，同步生成器函数，异步生成器函数，"
        "实例方法、类方法、静态方法}，因此不能被注入依赖"
    )



from beartype import BeartypeConf as _BeartypeConf
from beartype.door import is_bearable as _is_type
from beartype.door import is_subhint
from typing_extensions import Any, Awaitable, ParamSpec, Protocol, TypeIs, TypeVar

__all__ = ("AsyncCallable", "P", "T", "T_co", "is_type", "is_subhint")

#: 泛型 T，无约束
T = TypeVar("T", default=Any)
#: 泛型 T，无约束
U = TypeVar("U", default=Any)
#: 泛型 T，无约束
V = TypeVar("V", default=Any)
#: 泛型 T_co，协变无约束
T_co = TypeVar("T_co", covariant=True, default=Any)
#: :obj:`~typing.ParamSpec` 泛型 P，无约束
P = ParamSpec("P", default=Any)


class AsyncCallable(Protocol[P, T_co]):
    """用法：AsyncCallable[P, T]

    是该类型的等价形式：Callable[P, Awaitable[T]]
    """

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> Awaitable[T_co]: ...


class SyncOrAsyncCallable(Protocol[P, T_co]):
    """用法：SyncOrAsyncCallable[P, T]

    是该类型的等价形式：Callable[P, T | Awaitable[T]]
    """

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> T_co | Awaitable[T_co]: ...


_DEFAULT_BEARTYPE_CONF = _BeartypeConf(is_pep484_tower=True)


def is_type(obj: T, hint: type[Any]) -> TypeIs[T]:
    """检查 `obj` 是否是类型注解 `hint` 所表示的类型

    :param obj: 任意对象
    :param hint: 任意类型注解
    :return: 布尔值
    """
    ret = _is_type(obj, hint, conf=_DEFAULT_BEARTYPE_CONF)
    return ret  # type: ignore[no-any-return]



from melobot.typ import LogicMode
from melobot.utils.match import (
    ContainMatcher,
    EndMatcher,
    FullMatcher,
    RegexMatcher,
    StartMatcher,
)
from tests.base import *


async def test_matcher():
    assert await ContainMatcher("123").match("sadfa12345")
    assert not await ContainMatcher("123").match("ajdflja12s;djf3")
    assert await ContainMatcher(["12345", "asdjfajf;s"]).match("asdjf;aj12345asdfj")
    assert await ContainMatcher(["12345", "789"], LogicMode.AND).match(
        "123456sdf;da789as;jf"
    )

    assert await EndMatcher("123").match("sadfa123")
    assert not await EndMatcher("123").match("sadfa12345")

    assert await FullMatcher("123").match("123")
    assert not await FullMatcher("123").match("12345")

    assert await RegexMatcher(r"\d+").match("12345")
    assert not await RegexMatcher(r"[A-Z]+").match("abc123")

    assert await StartMatcher("123").match("123sadfa")
    assert not await StartMatcher("123").match("sadfa12345")



# pylint: disable=duplicate-code
import asyncio
import hmac
import json
import time
from asyncio import Future

import aiohttp
import aiohttp.log
import aiohttp.web

from melobot.io import SourceLifeSpan
from melobot.log import LogLevel

from .base import BaseIOSource
from .packet import EchoPacket, InPacket, OutPacket


class HttpIO(BaseIOSource):
    def __init__(
        self,
        onebot_host: str,
        onebot_port: int,
        serve_host: str,
        serve_port: int,
        secret: str | None = None,
        access_token: str | None = None,
        cd_time: float = 0,
    ) -> None:
        super().__init__(cd_time)
        self.onebot_url = f"http://{onebot_host}:{onebot_port}"
        self.host: str = serve_host
        self.port: int = serve_port
        self.serve_site: aiohttp.web.TCPSite
        self.client_session: aiohttp.ClientSession
        self.secret = secret
        self.access_token = access_token

        self._tasks: list[asyncio.Task] = []
        self._in_buf: asyncio.Queue[InPacket] = asyncio.Queue()
        self._out_buf: asyncio.Queue[OutPacket] = asyncio.Queue()
        self._pre_send_time = time.time_ns()

        self._echo_table: dict[str, tuple[str, Future[EchoPacket]]] = {}
        self._opened = asyncio.Event()
        self._lock = asyncio.Lock()

    async def _respond(self, request: aiohttp.web.Request) -> aiohttp.web.Response:
        if not self._opened.is_set():
            self._opened.set()

        data = await request.content.read()
        if data == b"":
            return aiohttp.web.Response(status=400)

        if self.secret is not None:
            sign = hmac.new(self.secret.encode(), data, "sha1").hexdigest()
            recv_sign = request.headers["X-Signature"][len("sha1=") :]

            if sign != recv_sign:
                self.logger.error("OneBot 实现程序鉴权不通过，本次上报数据将不会被处理")
                self.logger.generic_obj("试图上报的数据", data, level=LogLevel.ERROR)
                return aiohttp.web.Response(status=403)

        try:
            raw = json.loads(data.decode())
            if (
                self._hook_bus.get_evoke_time(SourceLifeSpan.STARTED) != -1
                and raw.get("post_type") == "meta_event"
                and raw.get("meta_event_type") == "lifecycle"
                and raw.get("sub_type") == "connect"
            ):
                await self._hook_bus.emit(SourceLifeSpan.RESTARTED, False)
            self.logger.generic_obj(
                "收到上报，未格式化的字典", str(raw), level=LogLevel.DEBUG
            )
            await self._in_buf.put(InPacket(time=raw["time"], data=raw))

        except Exception:
            self.logger.exception("OneBot v11 HTTP IO 源输入异常")
            self.logger.generic_obj("异常点的上报数据", raw, level=LogLevel.ERROR)

        finally:
            return (  # pylint: disable=return-in-finally,lost-exception
                aiohttp.web.Response(status=204)
            )

    async def _output_loop(self) -> None:
        while True:
            try:
                await self._opened.wait()

                out_packet = await self._out_buf.get()
                wait_time = self.cd_time - ((time.time_ns() - self._pre_send_time) / 1e9)
                await asyncio.sleep(wait_time)
                asyncio.create_task(self._handle_output(out_packet))
                self._pre_send_time = time.time_ns()

            except asyncio.CancelledError:
                break

            except Exception:
                self.logger.exception("OneBot v11 HTTP IO 源输出异常")
                self.logger.generic_obj("异常点局部变量", locals(), level=LogLevel.ERROR)

    async def _handle_output(self, packet: OutPacket) -> None:
        try:
            headers: dict | None = None
            if self.access_token is not None:
                headers = {"Authorization": f"Bearer {self.access_token}"}

            http_resp = await self.client_session.post(
                f"{self.onebot_url}/{packet.action_type}",
                json=packet.action_params,
                headers=headers,
            )
            if packet.echo_id is None:
                return

            raw = await http_resp.json()
            echo_id = raw.get("echo")
            if echo_id in (None, ""):
                return

            action_type, fut = self._echo_table.pop(echo_id)
            fut.set_result(
                EchoPacket(
                    time=int(time.time()),
                    data=raw,
                    ok=raw["status"] == "ok",
                    status=raw["retcode"],
                    action_type=action_type,
                )
            )
        except aiohttp.ContentTypeError:
            self.logger.error(
                "OneBot v11 HTTP IO 源无法解析上报数据。可能是 access_token 未配置或错误"
            )
        except Exception:
            self.logger.exception("OneBot v11 HTTP IO 源输出异常")
            self.logger.generic_obj("异常点的发送数据", packet.data, level=LogLevel.ERROR)

    async def open(self) -> None:
        if self.opened():
            return

        async with self._lock:
            if self.opened():
                return

            self.client_session = aiohttp.ClientSession()
            app = aiohttp.web.Application()
            app.add_routes([aiohttp.web.post("/", self._respond)])
            runner = aiohttp.web.AppRunner(app)

            await runner.setup()
            self.serve_site = aiohttp.web.TCPSite(runner, self.host, self.port)
            await self.serve_site.start()
            self._tasks.append(asyncio.create_task(self._output_loop()))

            self.logger.info("OneBot v11 HTTP IO 源就绪，等待实现端上线中")
            await self._opened.wait()
            self.logger.info("OneBot v11 HTTP IO 源双向通信已建立")

    def opened(self) -> bool:
        return self._opened.is_set()

    async def close(self) -> None:
        if not self.opened():
            return

        async with self._lock:
            if not self.opened():
                return

            await self.serve_site.stop()
            await self.client_session.close()
            for t in self._tasks:
                t.cancel()
            if len(self._tasks):
                await asyncio.wait(self._tasks)
            self._tasks.clear()

            self._opened.clear()
            self.logger.info("OneBot v11 HTTP IO 源已停止运行")

    async def input(self) -> InPacket:
        return await self._in_buf.get()

    async def output(self, packet: OutPacket) -> EchoPacket:
        await self._out_buf.put(packet)
        if packet.echo_id is None:
            return EchoPacket(noecho=True)

        fut: Future[EchoPacket] = asyncio.get_running_loop().create_future()
        self._echo_table[packet.echo_id] = (packet.action_type, fut)
        return await fut



from __future__ import annotations

import asyncio
import asyncio.tasks
import os
import platform
import sys
from contextlib import AsyncExitStack, ExitStack, asynccontextmanager, contextmanager
from enum import Enum
from os import PathLike
from pathlib import Path
from types import ModuleType

from typing_extensions import (
    Any,
    AsyncGenerator,
    Callable,
    Coroutine,
    Generator,
    Iterable,
    LiteralString,
    NoReturn,
)

from .._meta import MetaInfo
from ..adapter.base import Adapter
from ..ctx import BotCtx, LoggerCtx
from ..exceptions import BotError
from ..handle.base import Flow
from ..io.base import AbstractInSource, AbstractIOSource, AbstractOutSource
from ..log.base import GenericLogger, Logger, NullLogger
from ..mixin import HookMixin
from ..plugin.base import Plugin, PluginLifeSpan, PluginPlanner
from ..plugin.ipc import AsyncShare, IPCManager, SyncShare
from ..plugin.load import PluginLoader
from ..protocols.base import ProtocolStack
from ..typ.base import AsyncCallable, P, SyncOrAsyncCallable
from .dispatch import Dispatcher


class BotLifeSpan(Enum):
    """bot 生命周期阶段的枚举"""

    LOADED = "l"
    RELOADED = "r"
    STARTED = "sta"
    CLOSE = "c"
    STOPPED = "sto"


class BotExitSignal(Enum):
    NORMAL_STOP = 0
    ERROR = 1
    RESTART = 2


CLI_RUNTIME = "MELOBOT_CLI_RUNTIME"
CLI_DEV = "MELOBOT_CLI_DEV"
CLI_DEV_ALIVE_SIGNAL = "MELOBOT_CLI_DEV_ALIVE_SIGNAL"
LAST_EXIT_SIGNAL = "MELOBOT_LAST_EXIT_SIGNAL"
_BOT_CTX = BotCtx()
_LOGGER_CTX = LoggerCtx()


def _start_log(logger: GenericLogger) -> None:
    for row in MetaInfo.logo.split("\n"):
        logger.info(f"{row}")
    logger.info("")
    logger.info(f"版本：{MetaInfo.ver}")
    logger.info(f"系统：{platform.system()} {platform.release()} {platform.machine()}")
    logger.info(f"环境：{platform.python_implementation()} {platform.python_version()}")
    logger.info("=" * 40)


class Bot(HookMixin[BotLifeSpan]):
    """bot 类

    :ivar str name: bot 对象的名称
    :ivar GenericLogger logger: bot 对象使用的日志器
    """

    __instances__: dict[str, Bot] = {}

    def __new__(cls, name: str = "melobot", /, *args: Any, **kwargs: Any) -> Bot:
        if name in Bot.__instances__:
            raise BotError(f"命名为 {name} 的 bot 实例已存在，请改名避免冲突")
        obj = super().__new__(cls)
        Bot.__instances__[name] = obj
        return obj

    def __init__(
        self,
        name: str = "melobot",
        /,
        logger: GenericLogger | None = None,
        enable_log: bool = True,
    ) -> None:
        """
        初始化 bot

        :param name: bot 名称
        :param logger:
            bot 使用的日志器，符合 :class:`.GenericLogger` 的接口即可。
            可使用 melobot 内置的 :class:`.Logger`，或经过 :func:`.logger_patch` 修补的日志器
        :param enable_log: 是否启用日志功能
        """
        super().__init__(hook_type=BotLifeSpan, hook_tag=name)

        self.name = name
        self.logger: GenericLogger
        if not enable_log:
            self.logger = NullLogger()
        elif logger is None:
            self.logger = Logger()
        else:
            self.logger = logger

        self.adapters: dict[str, Adapter] = {}
        self.ipc_manager = IPCManager()

        self._in_srcs: dict[str, set[AbstractInSource]] = {}
        self._out_srcs: dict[str, set[AbstractOutSource]] = {}
        self._loader = PluginLoader()
        self._plugins: dict[str, Plugin] = {}
        self._dispatcher = Dispatcher()

        self._inited = False
        self._running = False
        self._rip_signal = asyncio.Event()

    def __repr__(self) -> str:
        return f'Bot(name="{self.name}")'

    @property
    def loop(self) -> asyncio.AbstractEventLoop:
        """获得当前 bot 运行时的事件循环对象"""
        return asyncio.get_running_loop()

    @contextmanager
    def _common_sync_ctx(self) -> Generator[ExitStack, None, None]:
        with ExitStack() as stack:
            stack.enter_context(_BOT_CTX.unfold(self))
            stack.enter_context(_LOGGER_CTX.unfold(self.logger))
            yield stack

    @asynccontextmanager
    async def _common_async_ctx(self) -> AsyncGenerator[AsyncExitStack, None]:
        async with AsyncExitStack() as stack:
            stack.enter_context(_BOT_CTX.unfold(self))
            stack.enter_context(_LOGGER_CTX.unfold(self.logger))
            yield stack

    def add_input(self, src: AbstractInSource) -> Bot:
        """绑定输入源

        :param src: 输入源
        :return: bot 对象，因此支持链式调用
        """
        if self._inited:
            raise BotError(f"{self} 已不在初始化期，无法再绑定输入源")

        self._in_srcs.setdefault(src.protocol, set()).add(src)
        return self

    def add_output(self, src: AbstractOutSource) -> Bot:
        """绑定输出源

        :param src: 输出源
        :return: bot 对象，因此支持链式调用
        """
        if self._inited:
            raise BotError(f"{self} 已不在初始化期，无法再绑定输出源")

        self._out_srcs.setdefault(src.protocol, set()).add(src)
        return self

    def add_io(self, src: AbstractIOSource) -> Bot:
        """绑定输入输出源

        :param src: 输入输出源
        :return: bot 对象，因此支持链式调用
        """
        self.add_input(src)
        self.add_output(src)
        return self

    def add_adapter(self, adapter: Adapter) -> Bot:
        """绑定适配器

        :param adapter: 适配器对象
        :return: bot 对象，因此支持链式调用
        """
        if self._inited:
            raise BotError(f"{self} 已不在初始化期，无法再绑定适配器")
        if adapter.protocol in self.adapters:
            raise BotError(
                f"已存在协议 {adapter.protocol} 的适配器，同协议的适配器不能再绑定"
            )

        self.adapters[adapter.protocol] = adapter
        return self

    def add_protocol(self, pstack: ProtocolStack) -> Bot:
        """绑定完整的协议栈，这包含了一组协同工作的输入源、输出源和适配器

        :param pstack: 协议栈对象
        :return: bot 对象，因此支持链式调用
        """
        insrcs, outsrcs, adapter = pstack.inputs, pstack.outputs, pstack.adapter
        self.add_adapter(adapter)
        for isrc in insrcs:
            self.add_input(isrc)
        for osrc in outsrcs:
            self.add_output(osrc)
        return self

    def _core_init(self) -> None:
        _start_log(self.logger)

        for protocol, srcs in self._in_srcs.items():
            for isrc in srcs:
                adapter = self.adapters.get(protocol)
                if adapter is not None:
                    adapter.in_srcs.add(isrc)
                else:
                    self.logger.warning(
                        f"输入源 {isrc.__class__.__name__} 没有对应的适配器"
                    )

        for protocol, outsrcs in self._out_srcs.items():
            for osrc in outsrcs:
                adapter = self.adapters.get(protocol)
                if adapter is not None:
                    adapter.out_srcs.add(osrc)
                else:
                    self.logger.warning(
                        f"输出源 {osrc.__class__.__name__} 没有对应的适配器"
                    )

        for adapter in self.adapters.values():
            if not len(adapter.in_srcs) and not len(adapter.out_srcs):
                self.logger.warning(
                    f"适配器 {adapter.__class__.__name__} 没有对应的输入源或输出源"
                )
            adapter.dispatcher = self._dispatcher

        self._inited = True
        self.logger.debug("bot 核心组件初始化完成")
        self.logger.debug(
            f"当前事件循环策略：<{asyncio.get_event_loop_policy().__class__.__name__}>"
        )

    def load_plugin(
        self,
        plugin: ModuleType | str | PathLike[str] | PluginPlanner,
        load_depth: int = 1,
    ) -> Bot:
        """加载插件

        :param plugin: 可以被加载为插件的对象（插件目录对应的模块，插件的目录路径，可直接 import 包名称，插件管理器对象）
        :param load_depth:
            插件加载时的相对引用深度，默认值 1 只支持向上引用到插件目录一级。
            增加为 2 可以引用到插件目录的父目录一级，依此类推。
            此参数只在 `plugin` 参数为插件的目录路径时有效。
        :return: bot 对象，因此支持链式调用
        """
        if not self._inited:
            self._core_init()

        with self._common_sync_ctx():
            p, is_loaded = self._loader.load(plugin, load_depth)
            if is_loaded:
                return self

            self._plugins[p.name] = p
            for share in p.shares:
                self.ipc_manager.add(p.name, share)
            for func in p.funcs:
                self.ipc_manager.add_func(p.name, func)
            self.logger.info(f"成功加载插件：{p.name}")

            if self._hook_bus.get_evoke_time(BotLifeSpan.STARTED) != -1:
                asyncio.create_task(
                    p.hook_bus.emit(
                        PluginLifeSpan.INITED,
                        callback=lambda _, p=p: self._dispatcher.add(*p.init_flows),
                    )
                )
            return self

    def load_plugins(
        self,
        plugins: Iterable[ModuleType | str | PathLike[str] | PluginPlanner],
        load_depth: int = 1,
    ) -> None:
        """与 :func:`load_plugin` 行为类似，但是参数变为可迭代对象

        :param plugins: 可迭代对象，包含：可以被加载为插件的对象（插件目录对应的模块，插件的目录路径，插件对象）
        :param load_depth: 参见 :func:`load_plugin` 同名参数
        """
        for p in plugins:
            self.load_plugin(p, load_depth)

    def load_plugins_dir(self, pdir: str | PathLike[str], load_depth: int = 1) -> None:
        """与 :func:`load_plugin` 行为类似，但是参数变为插件目录的父目录，本方法可以加载单个目录下的多个插件

        :param pdir: 插件所在父目录的路径
        :param load_depth: 参见 :func:`load_plugin` 同名参数
        """
        parent_dir = Path(pdir).resolve()
        plugin_dirs: list[Path] = []

        for dirname in os.listdir(parent_dir):
            path = Path(parent_dir).joinpath(dirname)
            if path.is_dir() and path.parts[-1] != "__pycache__":
                plugin_dirs.append(path)

        self.load_plugins(plugin_dirs, load_depth)

    def load_plugins_dirs(
        self, pdirs: Iterable[str | PathLike[str]], load_depth: int = 1
    ) -> None:
        """与 :func:`load_plugins_dir` 行为类似，但是参数变为可迭代对象，每个元素为包含插件目录的父目录。
        本方法可以加载多个目录下的多个插件

        :param pdirs: 可迭代对象，包含：插件所在父目录的路径
        :param load_depth: 参见 :func:`load_plugin` 同名参数
        """
        for pdir in pdirs:
            self.load_plugins_dir(pdir, load_depth)

    async def core_run(self) -> None:
        """运行 bot 的方法，可以在异步事件循环中自由地使用

        若使用此方法，则需要自行管理异步事件循环
        """
        if not self._inited:
            self._core_init()

        if self._running:
            raise BotError(f"{self} 已在运行中，不能再次启动运行")
        self._running = True

        try:
            async with self._common_async_ctx() as stack:
                await self._hook_bus.emit(BotLifeSpan.LOADED)
                if (
                    LAST_EXIT_SIGNAL in os.environ
                    and int(os.environ[LAST_EXIT_SIGNAL]) == BotExitSignal.RESTART.value
                ):
                    await self._hook_bus.emit(BotLifeSpan.RELOADED)

                for p in self._plugins.values():
                    await p.hook_bus.emit(
                        PluginLifeSpan.INITED,
                        callback=lambda _, p=p: self._dispatcher.add(*p.init_flows),
                    )

                self._dispatcher.start()

                ts = tuple(
                    asyncio.create_task(
                        stack.enter_async_context(adapter.__adapter_launch__())
                    )
                    for adapter in self.adapters.values()
                )
                if len(ts):
                    await asyncio.wait(ts)

                await self._hook_bus.emit(BotLifeSpan.STARTED)
                await self._rip_signal.wait()

        finally:
            async with self._common_async_ctx() as stack:
                await self._hook_bus.emit(BotLifeSpan.STOPPED, True)
                self.logger.info(f"{self} 已安全停止运行")
                self._running = False

    def run(self, debug: bool = False) -> None:
        """安全地运行 bot 的阻塞方法，这适用于只运行单一 bot 的情况

        :param debug: 是否启用 :py:mod:`asyncio` 的调试模式，但是这不会更改 :py:mod:`asyncio` 日志器的日志等级
        """
        _safe_run(self.core_run(), debug)

    @classmethod
    def start(cls, *bots: Bot, debug: bool = False) -> None:
        """安全地同时运行多个 bot 的阻塞方法

        :param bots: 要运行的 bot 对象
        :param debug: 参见 :func:`run` 同名参数
        """

        async def bots_run() -> None:
            tasks = []
            for bot in bots:
                tasks.append(asyncio.create_task(bot.core_run()))
            try:
                if len(tasks):
                    await asyncio.wait(tasks)
            except asyncio.CancelledError:
                pass

        _safe_run(bots_run(), debug)

    async def close(self) -> None:
        """停止并关闭当前 bot"""
        if not self._running:
            raise BotError(f"{self} 未在运行中，不能停止运行")

        await self._hook_bus.emit(BotLifeSpan.CLOSE, True)
        self._rip_signal.set()

    async def restart(self) -> NoReturn:
        """重启当前 bot，需要通过模块运行模式启动 bot 主脚本：

        .. code:: shell

            python3 -m melobot run [*.py]
        """
        if CLI_RUNTIME not in os.environ:
            raise BotError(
                "启用重启功能，需要用以下命令运行 bot：python -m melobot run [*.py]"
            )

        await self.close()
        sys.exit(BotExitSignal.RESTART.value)

    def get_adapter(
        self,
        type: LiteralString | type[Adapter] | None = None,
        filter: Callable[[Adapter], bool] | None = None,
    ) -> Adapter | None:
        """获取 bot 所绑定的适配器

        :param type: 适配器的类型（可传入协议字符串或协议类型），为空时才使用 `filter` 参数
        :param filter: 过滤函数，返回 `True` 则表明需要该适配器。为空则不使用
        :return: 适配器或空
        """
        if type is not None:
            if isinstance(type, str):
                return self.adapters.get(type)

            for a in self.adapters.values():
                if isinstance(a, type):
                    return a
            return None

        if filter is not None:
            for adapter in self.adapters.values():
                if filter(adapter):
                    return adapter

        return None

    def get_adapters(
        self, filter: Callable[[Adapter], bool] | None = None
    ) -> set[Adapter]:
        """获取一组适配器

        :param filter: 参见 :func:`get_adapter` 同名参数。但此处为空时直接获取所有适配器
        :return: 适配器的集合
        """
        if filter is None:
            return set(self.adapters.values())
        return set(adapter for adapter in self.adapters.values() if filter(adapter))

    def get_plugins(self) -> list[str]:
        """获取所有绑定的插件的名称

        :return: 所绑定的插件的名称列表
        """
        return list(self._plugins.keys())

    def get_share(self, plugin: str, share: str) -> SyncShare | AsyncShare:
        """获取绑定的插件中的共享对象

        :param plugin: 插件名称
        :param share: 共享对象的标识
        :return: 共享对象
        """
        return self.ipc_manager.get(plugin, share)

    def add_flows(self, *flows: Flow) -> None:
        """添加处理流

        :param flows: 流对象
        """
        if self._hook_bus.get_evoke_time(BotLifeSpan.STARTED) == -1:
            raise BotError(f"只有在 {BotLifeSpan.STARTED} 生命周期后才能动态添加处理流")
        self._dispatcher.add(*flows)

    @property
    def on_loaded(
        self,
    ) -> Callable[[SyncOrAsyncCallable[P, None]], AsyncCallable[P, None]]:
        """给 bot 注册 :obj:`.BotLifeSpan.LOADED` 阶段 hook 的装饰器"""
        return self.on(BotLifeSpan.LOADED)

    @property
    def on_reloaded(
        self,
    ) -> Callable[[SyncOrAsyncCallable[P, None]], AsyncCallable[P, None]]:
        """给 bot 注册 :obj:`.BotLifeSpan.RELOADED` 阶段 hook 的装饰器"""
        return self.on(BotLifeSpan.RELOADED)

    @property
    def on_started(
        self,
    ) -> Callable[[SyncOrAsyncCallable[P, None]], AsyncCallable[P, None]]:
        """给 bot 注册 :obj:`.BotLifeSpan.STARTED` 阶段 hook 的装饰器"""
        return self.on(BotLifeSpan.STARTED)

    @property
    def on_close(
        self,
    ) -> Callable[[SyncOrAsyncCallable[P, None]], AsyncCallable[P, None]]:
        """给 bot 注册 :obj:`.BotLifeSpan.CLOSE` 阶段 hook 的装饰器"""
        return self.on(BotLifeSpan.CLOSE)

    @property
    def on_stopped(
        self,
    ) -> Callable[[SyncOrAsyncCallable[P, None]], AsyncCallable[P, None]]:
        """给 bot 注册 :obj:`.BotLifeSpan.STOPPED` 阶段 hook 的装饰器"""
        return self.on(BotLifeSpan.STOPPED)


def _safe_run(main: Coroutine[Any, Any, None], debug: bool) -> None:
    try:
        loop = asyncio.get_event_loop()
        asyncio.get_event_loop_policy().set_event_loop(loop)

        if debug is not None:
            loop.set_debug(debug)

        if CLI_DEV in os.environ:
            main = alive_signal_guard(main)

        loop.run_until_complete(main)

    except KeyboardInterrupt:
        pass
    except asyncio.CancelledError:
        pass

    finally:
        try:
            _cancel_all_tasks(loop)
            loop.run_until_complete(loop.shutdown_asyncgens())
            loop.run_until_complete(loop.shutdown_default_executor())
        finally:
            asyncio.get_event_loop_policy().set_event_loop(None)
            loop.close()


async def alive_signal_guard(main_coro: Coroutine[Any, Any, None]) -> None:
    asyncio.create_task(main_coro)

    while True:
        if not os.path.exists(os.environ[CLI_DEV_ALIVE_SIGNAL]):
            for bot in Bot.__instances__.values():
                bot._rip_signal.set()
            break
        await asyncio.sleep(0.45)


def _cancel_all_tasks(loop: asyncio.AbstractEventLoop) -> None:
    to_cancel = asyncio.tasks.all_tasks(loop)
    if not to_cancel:
        return
    for task in to_cancel:
        task.cancel()
    loop.run_until_complete(asyncio.tasks.gather(*to_cancel, return_exceptions=True))

    for task in to_cancel:
        if task.cancelled():
            continue
        if task.exception() is not None:
            loop.call_exception_handler(
                {
                    "message": "unhandled exception during eventloop shutdown",
                    "exception": task.exception(),
                    "task": task,
                }
            )



from abc import abstractmethod

from typing_extensions import Any

from ...typ.cls import BetterABC, abstractattr


class AbstractParseArgs:
    """解析参数抽象类

    子类需要把以下属性按 :func:`.abstractattr` 的要求实现
    """

    vals: Any = abstractattr()
    """解析值

       :meta hide-value:
    """


class Parser(BetterABC):
    """解析器基类

    解析器一般用作从消息文本中按规则批量提取参数
    """

    def __init__(self) -> None:
        super().__init__()

    @abstractmethod
    async def parse(self, text: str) -> AbstractParseArgs | None:
        """解析方法

        任何解析器应该实现此抽象方法

        :param text: 消息文本内容
        :return: 解析结果，为空代表没有有效的解析参数

        """
        raise NotImplementedError



from .base import BaseInSource, BaseIOSource, BaseOutSource, BaseSource
from .duplex_http import HttpIO
from .forward import ForwardWebSocketIO
from .packet import InPacket
from .reverse import ReverseWebSocketIO



from __future__ import annotations

import asyncio
from asyncio import Queue, Task, get_running_loop

from ..adapter.base import Event
from ..handle.base import Flow
from ..mixin import LogMixin


class Dispatcher(LogMixin):
    def __init__(self) -> None:
        self.first_chan: EventChannel | None = None

        self._pending_chans: list[EventChannel] = []

    def __repr__(self) -> str:
        counts: dict[int, str] = {}
        chan = self.first_chan
        while chan is not None:
            counts[chan.priority] = (
                f"[flows:{chan.flow_que.qsize()}, events:{chan.event_que.qsize()}]"
            )
            chan = chan.next
        return f"{self.__class__.__name__}({counts})"

    def _arrange_chan(self, chan: EventChannel) -> None:
        try:
            get_running_loop()
            asyncio.create_task(chan.run())
        except RuntimeError:
            self._pending_chans.append(chan)

    def add(self, *flows: Flow) -> None:
        for f in flows:
            lvl = f.priority

            if self.first_chan is None:
                self.first_chan = EventChannel(self, priority=lvl)
                self.first_chan.flow_que.put_nowait(f)

            elif lvl == self.first_chan.priority:
                self.first_chan.flow_que.put_nowait(f)

            elif lvl > self.first_chan.priority:
                chan = EventChannel(self, priority=lvl)
                chan.set_next(self.first_chan)
                self.first_chan = chan
                chan.flow_que.put_nowait(f)

            else:
                chan = self.first_chan
                while chan.next is not None and lvl <= chan.next.priority:
                    chan = chan.next

                if lvl == chan.priority:
                    chan.flow_que.put_nowait(f)
                else:
                    new_chan = EventChannel(self, priority=lvl)
                    chan_next = chan.next
                    new_chan.set_pre(chan)
                    new_chan.set_next(chan_next)
                    new_chan.flow_que.put_nowait(f)

            f._active = True

    def remove(self, *flows: Flow) -> None:
        for f in flows:
            f._active = False

    def update(self, priority: int, *flows: Flow) -> None:
        self.remove(*flows)
        for f in flows:
            f.priority = priority
        self.add(*flows)

    def broadcast(self, event: Event) -> None:
        if self.first_chan is not None:
            self.first_chan.event_que.put_nowait(event)
        else:
            self.logger.warning(f"没有任何可用的事件处理流，事件 {event.id} 将被丢弃")

    def start(self) -> None:
        for chan in self._pending_chans:
            asyncio.create_task(chan.run())
        self._pending_chans.clear()


class EventChannel:
    def __init__(self, owner: Dispatcher, priority: int) -> None:
        self.owner = owner
        self.event_que: Queue[Event] = Queue()
        self.flow_que: Queue[Flow] = Queue()
        self.priority = priority

        self.pre: EventChannel | None = None
        self.next: EventChannel | None = None

        self.owner._arrange_chan(self)

    def set_pre(self, pre: EventChannel | None) -> None:
        self.pre = pre
        if self.pre is not None:
            self.pre.next = self

    def set_next(self, next: EventChannel | None) -> None:
        self.next = next
        if self.next is not None:
            self.next.pre = self

    async def run(self) -> None:
        handle_tasks: list[Task] = []
        events: list[Event] = []
        valid_flows: list[Flow] = []

        while True:
            events.clear()
            if self.event_que.qsize() == 0:
                ev = await self.event_que.get()
                events.append(ev)
            for _ in range(self.event_que.qsize()):
                events.append(self.event_que.get_nowait())

            for ev in events:
                ev.flag_set_default(self.owner, self.owner, set())
                handle_tasks.clear()
                valid_flows.clear()

                if self.flow_que.qsize() == 0:
                    self._dispose(*events)
                    return

                for _ in range(self.flow_que.qsize()):
                    handled_fs: set[Flow] = ev.flag_get(self.owner, self.owner)
                    f = self.flow_que.get_nowait()
                    if f._active and f.priority == self.priority:
                        if f not in handled_fs:
                            handle_tasks.append(asyncio.create_task(f._handle(ev)))
                            handled_fs.add(f)
                        valid_flows.append(f)

                for f in valid_flows:
                    self.flow_que.put_nowait(f)
                if len(valid_flows):
                    coro = self._determine_spread(ev, handle_tasks)
                    asyncio.create_task(coro)
                else:
                    self._dispose(*events)
                    return

    def _dispose(self, *events: Event) -> None:
        if self.pre is not None:
            self.pre.set_next(self.next)

        if self.next is not None:
            for ev in events:
                self.next.event_que.put_nowait(ev)

        if self is self.owner.first_chan:
            self.owner.first_chan = self.next

    async def _determine_spread(self, ev: Event, handle_tasks: list[Task]) -> None:
        if not len(handle_tasks):
            if self.next is not None:
                self.next.event_que.put_nowait(ev)
            return

        await asyncio.wait(handle_tasks)
        if self.next is not None and ev.spread:
            self.next.event_que.put_nowait(ev)



from __future__ import annotations

from enum import Enum

from typing_extensions import Literal, Optional, cast

from melobot.typ import SyncOrAsyncCallable
from melobot.utils.check import Checker

from ..adapter.event import Event, GroupMessageEvent, MessageEvent
from ..adapter.segment import AtSegment


class LevelRole(int, Enum):
    """用户权限等级枚举"""

    OWNER = 1 << 4
    SU = 1 << 3
    WHITE = 1 << 2
    NORMAL = 1 << 1
    BLACK = 1


class GroupRole(int, Enum):
    """群权限等级枚举"""

    OWNER = 1 << 2
    ADMIN = 1 << 1
    MEMBER = 1
    NOT_IN_GROUP = 0


def get_level_role(checker: MsgChecker, event: MessageEvent) -> LevelRole:
    """获得消息事件对应的分级权限等级

    :param event: 消息事件
    :return: 分级权限等级
    """
    qid = event.user_id

    if qid in checker.black_list:
        return LevelRole.BLACK
    if qid == checker.owner:
        return LevelRole.OWNER
    if qid in checker.su_list:
        return LevelRole.SU
    if qid in checker.white_list:
        return LevelRole.WHITE
    return LevelRole.NORMAL


def get_group_role(event: MessageEvent) -> GroupRole:
    """获得消息事件对应的群权限等级

    :param event: 消息事件
    :return: 群权限等级
    """
    if not event.is_group():
        return cast(GroupRole, GroupRole.NOT_IN_GROUP)
    if event.sender.is_group_owner():
        return GroupRole.OWNER
    if event.sender.is_group_admin():
        return GroupRole.ADMIN
    return GroupRole.MEMBER


class MsgChecker(Checker[Event]):
    """消息事件分级权限检查器

    主要分 主人、超级用户、白名单用户、普通用户、黑名单用户 五级
    """

    def __init__(
        self,
        role: LevelRole | GroupRole,
        owner: Optional[int] = None,
        super_users: Optional[list[int]] = None,
        white_users: Optional[list[int]] = None,
        black_users: Optional[list[int]] = None,
        fail_cb: Optional[SyncOrAsyncCallable[[], None]] = None,
    ) -> None:
        """初始化一个消息事件分级权限检查器

        :param role: 允许的等级（>= 此等级才能通过校验）
        :param owner: 主人的 qq 号
        :param super_users: 超级用户 qq 号列表
        :param white_users: 白名单用户 qq 号列表
        :param black_users: 黑名单用户 qq 号列表
        :param fail_cb: 检查不通过的回调
        """
        super().__init__(fail_cb)
        self.check_role = role

        self.owner = owner
        self.su_list = super_users if super_users is not None else []
        self.white_list = white_users if white_users is not None else []
        self.black_list = black_users if black_users is not None else []

    def _get_level(self, event: MessageEvent) -> LevelRole | GroupRole:
        """获得事件对应的登记"""

        if isinstance(self.check_role, LevelRole):
            return get_level_role(self, event)
        return get_group_role(event)

    def _check(self, event: MessageEvent) -> bool:
        e_level = self._get_level(event)
        if isinstance(e_level, LevelRole):
            status = LevelRole.BLACK < e_level and e_level >= self.check_role
        else:
            status = e_level >= self.check_role
        return status

    async def check(self, event: Event) -> bool:
        # 不要使用 isinstace，避免通过反射模式注入的 event 依赖产生误判结果
        if not event.is_message():
            status = False
        else:
            status = self._check(cast(MessageEvent, event))

        if not status and self.fail_cb is not None:
            await self.fail_cb()
        return status


class GroupMsgChecker(MsgChecker):
    """群聊消息事件分级权限检查器

    基本功能与 :class:`MsgChecker` 一致。但增加了白名单机制，
    不提供 `white_groups` 参数默认拒绝所有群聊消息事件。

    对所有私聊消息事件校验不通过。
    """

    def __init__(
        self,
        role: LevelRole | GroupRole,
        owner: Optional[int] = None,
        super_users: Optional[list[int]] = None,
        white_users: Optional[list[int]] = None,
        black_users: Optional[list[int]] = None,
        white_groups: Optional[list[int]] = None,
        fail_cb: Optional[SyncOrAsyncCallable[[], None]] = None,
    ) -> None:
        """初始化一个群聊消息事件分级权限检查器

        :param role: 允许的等级（>= 此等级才能通过校验）
        :param owner: 主人的 qq 号
        :param super_users: 超级用户 qq 号列表
        :param white_users: 白名单用户 qq 号列表
        :param black_users: 黑名单用户 qq 号列表
        :param white_groups: 白名单群号列表（不在其中的群不通过校验）
        :param fail_cb: 检查不通过的回调
        """
        super().__init__(role, owner, super_users, white_users, black_users, fail_cb)
        self.white_group_list = white_groups if white_groups is not None else []

    def _check(self, event: MessageEvent) -> bool:
        # 不要使用 isinstace，避免通过反射模式注入的 event 依赖产生误判结果
        if event.is_private():
            return False
        if len(self.white_group_list) == 0:
            return False
        if cast(GroupMessageEvent, event).group_id not in self.white_group_list:
            return False

        return super()._check(event)


class PrivateMsgChecker(MsgChecker):
    """私聊消息事件分级权限检查器

    基本功能与 :class:`MsgChecker` 一致。

    对所有群聊消息事件校验不通过。
    """

    def __init__(
        self,
        role: LevelRole,
        owner: Optional[int] = None,
        super_users: Optional[list[int]] = None,
        white_users: Optional[list[int]] = None,
        black_users: Optional[list[int]] = None,
        fail_cb: Optional[SyncOrAsyncCallable[[], None]] = None,
    ) -> None:
        """初始化一个私聊消息事件分级权限检查器

        :param role: 允许的等级（>= 此等级才能通过校验）
        :param owner: 主人的 qq 号
        :param super_users: 超级用户 qq 号列表
        :param white_users: 白名单用户 qq 号列表
        :param black_users: 黑名单用户 qq 号列表
        :param fail_cb: 检查不通过的回调
        """
        super().__init__(role, owner, super_users, white_users, black_users, fail_cb)

    def _check(self, event: MessageEvent) -> bool:
        if not event.is_private():
            return False

        return super()._check(event)


class MsgCheckerFactory:
    """消息事件分级权限检查器的工厂

    预先存储检查依据（各等级数据），指定检查等级后，可返回一个 :class:`MsgChecker` 类的对象
    """

    def __init__(
        self,
        owner: Optional[int] = None,
        super_users: Optional[list[int]] = None,
        white_users: Optional[list[int]] = None,
        black_users: Optional[list[int]] = None,
        white_groups: Optional[list[int]] = None,
        fail_cb: Optional[SyncOrAsyncCallable[[], None]] = None,
    ) -> None:
        """初始化一个消息事件分级权限检查器的工厂

        :param owner: 主人的 qq 号
        :param super_users: 超级用户 qq 号列表
        :param white_users: 白名单用户 qq 号列表
        :param black_users: 黑名单用户 qq 号列表
        :param white_groups: 白名单群号列表（不在其中的群不通过校验）
        :param fail_cb: 检查不通过的回调（这将自动附加到生成的检查器上）
        """
        self.owner = owner
        self.su_list = super_users if super_users is not None else []
        self.white_list = white_users if white_users is not None else []
        self.black_list = black_users if black_users is not None else []
        self.white_group_list = white_groups if white_groups is not None else []

        self.fail_cb = fail_cb

    def get_base(
        self,
        role: LevelRole | GroupRole,
        fail_cb: Optional[SyncOrAsyncCallable[[], None]] = None,
    ) -> MsgChecker:
        """根据内部依据和给定等级，生成一个 :class:`MsgChecker` 对象

        :param role: 允许的等级（>= 此等级才能通过校验）
        :param fail_cb: 检查不通过的回调
        :return: 消息事件分级权限检查器
        """
        return MsgChecker(
            role,
            self.owner,
            self.su_list,
            self.white_list,
            self.black_list,
            self.fail_cb if fail_cb is None else fail_cb,
        )

    def get_group(
        self,
        role: LevelRole | GroupRole,
        fail_cb: Optional[SyncOrAsyncCallable[[], None]] = None,
    ) -> GroupMsgChecker:
        """根据内部依据和给定等级，生成一个 :class:`GroupMsgChecker` 对象

        :param role: 允许的等级（>= 此等级才能通过校验）
        :param fail_cb: 检查不通过的回调
        :return: 群聊消息事件分级权限检查器
        """
        return GroupMsgChecker(
            role,
            self.owner,
            self.su_list,
            self.white_list,
            self.black_list,
            self.white_group_list,
            self.fail_cb if fail_cb is None else fail_cb,
        )

    def get_private(
        self,
        role: LevelRole,
        fail_cb: Optional[SyncOrAsyncCallable[[], None]] = None,
    ) -> PrivateMsgChecker:
        """根据内部依据和给定等级，生成一个 :class:`PrivateMsgChecker` 对象

        :param role: 允许的等级（>= 此等级才能通过校验）
        :param fail_cb: 检查不通过的回调
        :return: 私聊消息事件分级权限检查器
        """
        return PrivateMsgChecker(
            role,
            self.owner,
            self.su_list,
            self.white_list,
            self.black_list,
            self.fail_cb if fail_cb is None else fail_cb,
        )


class AtMsgChecker(Checker):
    """艾特消息事件检查器"""

    def __init__(
        self,
        qid: int | Literal["all"] | None = None,
        fail_cb: Optional[SyncOrAsyncCallable[[], None]] = None,
    ) -> None:
        """初始化一个艾特消息事件检查器

        :param qid: 被艾特的 qq 号。为空则接受所有艾特消息事件；不为空则只接受指定 qid 被艾特的艾特消息事件
        :param fail_cb: 检查不通过的回调
        """
        super().__init__(fail_cb)
        self.qid = qid

    async def check(self, event: Event) -> bool:
        # 不要使用 isinstace，避免通过反射模式注入的 event 依赖产生误判结果
        if not event.is_message():
            return False

        event = cast(MessageEvent, event)
        qids = [seg.data["qq"] for seg in event.message if isinstance(seg, AtSegment)]
        if self.qid is None:
            return len(qids) > 0
        status = any(id == self.qid for id in qids)

        if not status and self.fail_cb is not None:
            await self.fail_cb()
        return status



import asyncio
import time

from typing_extensions import Any, Callable, Coroutine

from ..typ.base import T


def call_later(callback: Callable[[], None], delay: float) -> asyncio.TimerHandle:
    """同步函数延迟调度

    在指定的 `delay` 后调度一个 `callback` 执行。`callback` 应该是同步方法。

    :param callback: 同步函数
    :param delay: 多长时间后调度
    :return: :class:`asyncio.TimerHandle` 对象
    """
    return asyncio.get_running_loop().call_later(delay, callback)


def call_at(callback: Callable[[], None], timestamp: float) -> asyncio.TimerHandle:
    """同步函数指定时间调度

    在指定的时间戳调度一个 `callback` 执行。`callback` 应该是同步方法。`timestamp` <= 当前时刻回调立即执行

    :param callback: 同步函数
    :param timestamp: 在什么时刻调度
    :return: :class:`asyncio.TimerHandle` 对象
    """
    loop = asyncio.get_running_loop()
    if timestamp <= time.time_ns() / 1e9:
        return loop.call_later(0, callback)

    return loop.call_later(timestamp - time.time_ns() / 1e9, callback)


def async_later(callback: Coroutine[Any, Any, T], delay: float) -> asyncio.Task[T]:
    """异步函数延迟调度（可自主选择是否等待）

    在指定的 `delay` 后调度一个 `callback` 执行。`callback` 是协程。

    返回一个 :class:`asyncio.Task` 对象，等待 :class:`asyncio.Task` 即是等待 `callback` 的返回值。

    :param callback: 协程（可有返回值）
    :param delay: 多长时间后调度
    :return: :class:`asyncio.Task` 对象
    """

    async def _later_task() -> T:
        try:
            await asyncio.sleep(delay)
            res = await callback
            return res
        except asyncio.CancelledError:
            callback.close()
            raise

    return asyncio.create_task(_later_task())


def async_at(callback: Coroutine[Any, Any, T], timestamp: float) -> asyncio.Task[T]:
    """异步函数指定时间调度（可自主选择是否等待）

    在指定的时间戳调度一个 `callback` 执行。`callback` 是协程。

    返回一个 :class:`asyncio.Task` 对象，等待 :class:`asyncio.Task` 即是等待 `callback` 的返回值。

    注意：如果 `callback` 未完成就被取消，需要捕获 :class:`asyncio.CancelledError`。

    :param callback: 协程（可有返回值）
    :param timestamp: 在什么时刻调度
    :return: :class:`asyncio.Task` 对象
    """
    if timestamp <= time.time_ns() / 1e9:
        return async_later(callback, 0)

    return async_later(callback, timestamp - time.time_ns() / 1e9)


def async_interval(
    callback: Callable[[], Coroutine[Any, Any, None]], interval: float
) -> asyncio.Task[None]:
    """异步函数间隔调度（类似 JavaScript 的 setInterval）

    每过时间间隔执行 `callback` 一次。`callback` 是返回协程的可调用对象（异步函数或 lambda 函数等）。

    返回一个 :class:`asyncio.Task` 对象，可使用该 task 取消调度过程。

    :param callback: 异步函数
    :param interval: 调度的间隔
    :return: :class:`asyncio.Task` 对象
    """

    async def _interval_task() -> None:
        try:
            while True:
                coro = callback()
                await asyncio.sleep(interval)
                await coro
        except asyncio.CancelledError:
            coro.close()
            raise

    return asyncio.create_task(_interval_task())



from __future__ import annotations

import time
from abc import abstractmethod
from dataclasses import dataclass, field
from enum import Enum
from types import TracebackType

from typing_extensions import Any, Generic, LiteralString, Self, TypeVar

from ..mixin import HookMixin, LogMixin
from ..typ.cls import BetterABC, abstractattr
from ..utils.common import get_id


@dataclass
class InPacket:
    """输入包基类（数据类）

    :ivar float time: 时间戳
    :ivar str id: id 标识
    :ivar typing.LiteralString | None protocol: 遵循的协议
    :ivar Any data: 附加的数据
    """

    time: float = field(default_factory=lambda: time.time_ns() / 1e9)
    id: str = field(default_factory=get_id)
    protocol: LiteralString | None = None
    data: Any = None


@dataclass
class OutPacket:
    """输出包基类（数据类）

    :ivar float time: 时间戳
    :ivar str id: id 标识
    :ivar typing.LiteralString | None protocol: 遵循的协议
    :ivar Any data: 附加的数据
    """

    time: float = field(default_factory=lambda: time.time_ns() / 1e9)
    id: str = field(default_factory=get_id)
    protocol: LiteralString | None = None
    data: Any = None


@dataclass
class EchoPacket:
    """回应包基类（数据类）

    :ivar float time: 时间戳
    :ivar str id: id 标识
    :ivar typing.LiteralString | None protocol: 遵循的协议
    :ivar Any data: 附加的数据
    :ivar bool ok: 回应是否成功
    :ivar int status: 回应状态码
    :ivar str prompt: 提示语
    :ivar bool noecho: 是否并无回应产生
    """

    time: float = field(default_factory=lambda: time.time_ns() / 1e9)
    id: str = field(default_factory=get_id)
    protocol: LiteralString | None = None
    data: Any = None
    ok: bool = True
    status: int = 0
    prompt: str = ""
    noecho: bool = False


InPacketT = TypeVar("InPacketT", bound=InPacket)
OutPacketT = TypeVar("OutPacketT", bound=OutPacket)
EchoPacketT = TypeVar("EchoPacketT", bound=EchoPacket)


class SourceLifeSpan(Enum):
    """源生命周期阶段的枚举"""

    STARTED = "sta"
    RESTARTED = "res"
    STOPPED = "sto"


class AbstractSource(HookMixin[SourceLifeSpan], LogMixin, BetterABC):
    """抽象源基类"""

    protocol: LiteralString = abstractattr()

    def __init__(self) -> None:
        super().__init__(
            hook_type=SourceLifeSpan,
            hook_tag=f"{self.__class__.__module__}.{self.__class__.__name__}",
        )

    @abstractmethod
    async def open(self) -> None:
        """源打开方法"""
        raise NotImplementedError

    @abstractmethod
    def opened(self) -> bool:
        """源是否已打开"""
        raise NotImplementedError

    @abstractmethod
    async def close(self) -> None:
        """源关闭方法"""
        raise NotImplementedError

    async def __aenter__(self) -> Self:
        if self.opened():
            return self

        await self.open()
        await self._hook_bus.emit(SourceLifeSpan.STARTED)
        return self

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None:
        if not self.opened():
            return None

        try:
            await self.close()
        finally:
            await self._hook_bus.emit(SourceLifeSpan.STOPPED, True)
        return None


class AbstractInSource(AbstractSource, Generic[InPacketT]):
    """抽象输入源基类"""

    @abstractmethod
    async def open(self) -> None:
        raise NotImplementedError

    @abstractmethod
    def opened(self) -> bool:
        raise NotImplementedError

    @abstractmethod
    async def close(self) -> None:
        raise NotImplementedError

    @abstractmethod
    async def input(self) -> InPacketT:
        """源输入方法

        :return: 返回 :class:`.InPacket` 对象
        """
        raise NotImplementedError


InSourceT = TypeVar("InSourceT", bound=AbstractInSource)


class AbstractOutSource(AbstractSource, Generic[OutPacketT, EchoPacketT]):
    """抽象输出源基类"""

    @abstractmethod
    async def open(self) -> None:
        raise NotImplementedError

    @abstractmethod
    def opened(self) -> bool:
        raise NotImplementedError

    @abstractmethod
    async def close(self) -> None:
        raise NotImplementedError

    @abstractmethod
    async def output(self, packet: OutPacketT) -> EchoPacketT:
        """源输出方法

        :return: 返回 :class:`.OutPacket` 对象
        """
        raise NotImplementedError


OutSourceT = TypeVar("OutSourceT", bound=AbstractOutSource)

InOrOutSourceT = TypeVar("InOrOutSourceT", bound=AbstractInSource | AbstractOutSource)


class AbstractIOSource(
    AbstractInSource[InPacketT], AbstractOutSource[OutPacketT, EchoPacketT]
):
    """抽象输入输出源基类"""

    @abstractmethod
    async def open(self) -> None:
        raise NotImplementedError

    @abstractmethod
    async def close(self) -> None:
        raise NotImplementedError

    @abstractmethod
    def opened(self) -> bool:
        raise NotImplementedError

    @abstractmethod
    async def input(self) -> InPacketT:
        raise NotImplementedError

    @abstractmethod
    async def output(self, packet: OutPacketT) -> EchoPacketT:
        raise NotImplementedError


IOSourceT = TypeVar("IOSourceT", bound=AbstractIOSource)



import asyncio
import inspect
import time
from functools import wraps

from typing_extensions import Any, AsyncContextManager, Callable, ContextManager, cast

from ..exceptions import UtilValidateError
from ..typ.base import AsyncCallable, P, SyncOrAsyncCallable, T, U, V
from .base import to_async


def if_not(
    condition: SyncOrAsyncCallable[[], U] | U,
    reject: SyncOrAsyncCallable[[], None],
    give_up: bool = False,
    accept: SyncOrAsyncCallable[[U], None] | None = None,
) -> Callable[[SyncOrAsyncCallable[P, T]], AsyncCallable[P, T | None]]:
    """条件判断装饰器

    :param condition: 用于判断的条件（如果是可调用对象，则先求值再转为 bool 值）
    :param reject: 当条件为 `False` 时，执行的回调
    :param give_up: 在条件为 `False` 时，是否放弃执行被装饰函数
    :param accept: 当条件为 `True` 时，执行的回调
    """
    _condition = to_async(condition) if callable(condition) else condition
    _reject = to_async(reject)
    _accept = to_async(accept) if accept is not None else accept

    def if_not_wrapper(func: SyncOrAsyncCallable[P, T]) -> AsyncCallable[P, T | None]:
        _func = to_async(func)

        @wraps(func)
        async def if_not_wrapped(*args: P.args, **kwargs: P.kwargs) -> T | None:
            if not callable(_condition):
                cond = _condition
            else:
                obj = _condition()
                cond = await obj if inspect.isawaitable(obj) else obj

            if not cond:
                await _reject()

            if cond or not give_up:
                if _accept is not None:
                    await _accept(cond)

                return await _func(*args, **kwargs)
            return None

        return if_not_wrapped

    return if_not_wrapper


def unfold_ctx(
    getter: SyncOrAsyncCallable[[], ContextManager | AsyncContextManager],
) -> Callable[[SyncOrAsyncCallable[P, T]], AsyncCallable[P, T]]:
    """上下文装饰器

    展开一个上下文，供被装饰函数使用。
    但注意此装饰器不支持获取上下文管理器 `yield` 的值

    :param getter: 上下文管理器或上下文管理器获取方法
    """

    _getter = to_async(getter)

    def unfold_ctx_wrapper(func: SyncOrAsyncCallable[P, T]) -> AsyncCallable[P, T]:
        _func = to_async(func)

        @wraps(func)
        async def unfold_ctx_wrapped(*args: P.args, **kwargs: P.kwargs) -> T:
            try:
                manager = await _getter()
            except Exception as e:
                raise UtilValidateError(
                    f"{unfold_ctx.__name__} 的 getter 参数为：{getter}，调用它获取上下文管理器失败：{e}"
                ) from e

            if isinstance(manager, ContextManager):
                with manager:
                    return await _func(*args, **kwargs)
            elif isinstance(manager, AsyncContextManager):
                async with manager:
                    return await _func(*args, **kwargs)
            else:
                raise UtilValidateError(
                    f"{unfold_ctx.__name__} 的 getter 参数为：{getter}，调用它返回了无效的上下文管理器"
                )

        return unfold_ctx_wrapped

    return unfold_ctx_wrapper


def lock(
    callback: SyncOrAsyncCallable[[], U] | None = None
) -> Callable[[SyncOrAsyncCallable[P, T]], AsyncCallable[P, T | U]]:
    """锁装饰器

    本方法作为异步函数的装饰器使用，可以为被装饰函数加锁。

    在获取锁冲突时，调用 `callback` 获得一个回调并执行。回调执行完毕后直接返回。

    `callback` 参数为空，只应用 :class:`asyncio.Lock` 的锁功能。

    被装饰函数的返回值：被装饰函数被执行 -> 被装饰函数返回值；执行任何回调 -> 那个回调的返回值

    :param callback: 获取锁冲突时的回调
    """
    alock = asyncio.Lock()
    _callback = to_async(callback) if callback is not None else None

    def lock_wrapper(func: SyncOrAsyncCallable[P, T]) -> AsyncCallable[P, T | U]:
        _func = to_async(func)

        @wraps(func)
        async def lock_wrapped(*args: P.args, **kwargs: P.kwargs) -> T | U:
            if _callback is not None and alock.locked():
                return await _callback()
            async with alock:
                return await _func(*args, **kwargs)

        return lock_wrapped

    return lock_wrapper


def cooldown(
    busy_callback: SyncOrAsyncCallable[[], U] | None = None,
    cd_callback: SyncOrAsyncCallable[[float], V] | None = None,
    interval: float = 5,
) -> Callable[[SyncOrAsyncCallable[P, T]], AsyncCallable[P, T | U | V]]:
    """冷却装饰器

    本方法作为异步函数的装饰器使用，可以为被装饰函数添加 cd 时间。

    如果被装饰函数已有一个在运行，此时调用 `busy_callback` 生成回调并执行。回调执行完毕后直接返回。

    `busy_callback` 参数为空，则等待已运行的运行完成。随后执行下面的“冷却”处理逻辑。

    当被装饰函数没有在运行的，但冷却时间未结束：
        - `cd_callback` 不为空：使用 `cd_callback` 生成回调并执行。
        - `cd_callback` 为空，被装饰函数持续等待，直至冷却结束再执行。

    被装饰函数的返回值：被装饰函数被执行 -> 被装饰函数返回值；执行任何回调 -> 那个回调的返回值

    :param busy_callback: 已运行时的回调
    :param cd_callback: 冷却时间未结束的回调
    :param interval: 冷却时间
    """
    alock = asyncio.Lock()
    pre_finish_t = time.perf_counter() - interval - 1

    _busy_callback = to_async(busy_callback) if busy_callback is not None else None
    _cd_callback = to_async(cd_callback) if cd_callback is not None else None

    def cooldown_wrapper(func: SyncOrAsyncCallable[P, T]) -> AsyncCallable[P, T | U | V]:
        _func = to_async(func)

        @wraps(func)
        async def cooldown_wrapped(*args: P.args, **kwargs: P.kwargs) -> T | U | V:
            nonlocal pre_finish_t
            if _busy_callback is not None and alock.locked():
                return await _busy_callback()

            async with alock:
                duration = time.perf_counter() - pre_finish_t
                if duration > interval:
                    ret = await _func(*args, **kwargs)
                    pre_finish_t = time.perf_counter()
                    return ret

                remain_t = interval - duration
                if _cd_callback is not None:
                    return await _cd_callback(remain_t)

                await asyncio.sleep(remain_t)
                ret = await _func(*args, **kwargs)
                pre_finish_t = time.perf_counter()
                return ret

        return cooldown_wrapped

    return cooldown_wrapper


def semaphore(
    callback: SyncOrAsyncCallable[[], U] | None = None, value: int = -1
) -> Callable[[SyncOrAsyncCallable[P, T]], AsyncCallable[P, T | U]]:
    """信号量装饰器

    本方法作为异步函数的装饰器使用，可以为被装饰函数添加信号量控制。

    在信号量无法立刻获取时，将调用 `callback` 获得回调并执行。回调执行完毕后直接返回。

    `callback` 参数为空，只应用 :class:`asyncio.Semaphore` 的信号量功能。

    被装饰函数的返回值：被装饰函数被执行 -> 被装饰函数返回值；执行任何回调 -> 那个回调的返回值

    :param callback: 信号量无法立即获取的回调
    :param value: 信号量阈值
    """
    a_semaphore = asyncio.Semaphore(value)
    _callback = to_async(callback) if callback is not None else None

    def semaphore_wrapper(func: SyncOrAsyncCallable[P, T]) -> AsyncCallable[P, T | U]:
        _func = to_async(func)

        @wraps(func)
        async def semaphore_wrapped(*args: P.args, **kwargs: P.kwargs) -> T | U:
            if _callback is not None and a_semaphore.locked():
                return await _callback()
            async with a_semaphore:
                return await _func(*args, **kwargs)

        return semaphore_wrapped

    return semaphore_wrapper


def timelimit(
    callback: SyncOrAsyncCallable[[], U] | None = None, timeout: float = 5
) -> Callable[[SyncOrAsyncCallable[P, T]], AsyncCallable[P, T | U]]:
    """时间限制装饰器

    本方法作为异步函数的装饰器使用，可以为被装饰函数添加超时控制。

    超时之后，调用 `callback` 获得回调并执行，同时取消原任务。

    `callback` 参数为空，如果超时，则抛出 :class:`asyncio.TimeoutError` 异常。

    被装饰函数的返回值：被装饰函数被执行 -> 被装饰函数返回值；执行任何回调 -> 那个回调的返回值

    :param callback: 超时时的回调
    :param timeout: 超时时间
    """
    _callback = to_async(callback) if callback is not None else None

    def timelimit_wrapper(func: SyncOrAsyncCallable[P, T]) -> AsyncCallable[P, T | U]:
        _func = to_async(func)

        @wraps(func)
        async def timelimit_wrapped(*args: P.args, **kwargs: P.kwargs) -> T | U:
            try:
                return await asyncio.wait_for(_func(*args, **kwargs), timeout)
            except asyncio.TimeoutError:
                if _callback is None:
                    raise TimeoutError("timelimit 所装饰的任务已超时") from None
                return await _callback()

        return timelimit_wrapped

    return timelimit_wrapper


def speedlimit(
    callback: SyncOrAsyncCallable[[], U] | None = None,
    limit: int = 60,
    duration: int = 60,
) -> Callable[[SyncOrAsyncCallable[P, T]], AsyncCallable[P, T | U]]:
    """流量/速率限制装饰器（使用固定窗口算法）

    本方法作为异步函数的装饰器使用，可以为被装饰函数添加流量控制：`duration` 秒内只允许 `limit` 次调用。

    超出调用速率限制后，调用 `callback` 获得回调并执行，同时取消原任务。

    `callback` 参数为空，等待直至满足速率控制要求再调用。

    被装饰函数的返回值：被装饰函数被执行 -> 被装饰函数返回值；执行任何回调 -> 那个回调的返回值。

    :param callback: 超出速率限制时的回调
    :param limit: `duration` 秒内允许调用多少次
    :param duration: 时长区间
    """
    called_num = 0
    min_start = time.perf_counter()
    if limit <= 0:
        raise UtilValidateError("speedlimit 装饰器的 limit 参数必须 > 0")
    if duration <= 0:
        raise UtilValidateError("speedlimit 装饰器的 duration 参数必须 > 0")

    _callback = to_async(callback) if callback is not None else None

    def speedlimit_wrapper(func: SyncOrAsyncCallable[P, T]) -> AsyncCallable[P, T | U]:
        _func = to_async(func)

        @wraps(func)
        async def speedlimit_wrapped(*args: P.args, **kwargs: P.kwargs) -> T | U:
            fut = _speedlimit_wrapped(_func, *args, **kwargs)
            fut = cast(asyncio.Future[T | U | Exception], fut)
            fut_ret = await fut
            if isinstance(fut_ret, Exception):
                raise fut_ret
            return fut_ret

        return speedlimit_wrapped

    def _speedlimit_wrapped(
        func: AsyncCallable[P, T],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> asyncio.Future:
        # 分离出来定义，方便 result_set 调用形成递归。主要逻辑通过 Future 实现，有利于避免竞争问题。
        nonlocal called_num, min_start
        passed_time = time.perf_counter() - min_start
        res_fut: Any = asyncio.get_running_loop().create_future()

        if passed_time <= duration:
            if called_num < limit:
                called_num += 1
                asyncio.create_task(
                    _speedlimit_set_result(func, res_fut, -1, *args, **kwargs)
                )

            elif _callback is not None:
                asyncio.create_task(_speedlimit_set_result(_callback, res_fut, -1))

            else:
                asyncio.create_task(
                    _speedlimit_set_result(
                        func, res_fut, duration - passed_time, *args, **kwargs
                    )
                )
        else:
            called_num, min_start = 0, time.perf_counter()
            called_num += 1
            asyncio.create_task(
                _speedlimit_set_result(func, res_fut, -1, *args, **kwargs)
            )

        return cast(asyncio.Future, res_fut)

    async def _speedlimit_set_result(
        func: AsyncCallable[P, T],
        fut: asyncio.Future,
        delay: float,
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> None:
        """
        只有依然在当前 duration 区间内，但超出调用次数限制的，需要等待。
        随后就是递归调用。delay > 0 为需要递归的分支。
        """
        nonlocal called_num
        try:
            if delay > 0:
                await asyncio.sleep(delay)
                res = await _speedlimit_wrapped(func, *args, **kwargs)
                fut.set_result(res)
                return

            res = await func(*args, **kwargs)
            fut.set_result(res)

        except Exception as e:
            fut.set_result(e)

    return speedlimit_wrapper



from types import ModuleType

from typing_extensions import Any

from melobot.utils.common import DeprecatedLoader as _DeprecatedLoader

from .check import (
    AtMsgChecker,
    GroupMsgChecker,
    GroupRole,
    LevelRole,
    MsgChecker,
    MsgCheckerFactory,
    PrivateMsgChecker,
    get_group_role,
    get_level_role,
)

_LOADER = _DeprecatedLoader(
    __name__,
    {
        "Checker": ("melobot.utils.check", "Checker", "3.1.1"),
        "Matcher": ("melobot.utils.match", "Matcher", "3.1.1"),
        "ParseArgs": ("melobot.utils.parse", "CmdArgs", "3.1.1"),
        "Parser": ("melobot.utils.parse", "Parser", "3.1.1"),
        "ContainMatcher": ("melobot.utils.match", "ContainMatcher", "3.1.1"),
        "EndMatcher": ("melobot.utils.match", "EndMatcher", "3.1.1"),
        "FullMatcher": ("melobot.utils.match", "FullMatcher", "3.1.1"),
        "RegexMatcher": ("melobot.utils.match", "RegexMatcher", "3.1.1"),
        "StartMatcher": ("melobot.utils.match", "StartMatcher", "3.1.1"),
        "CmdArgFormatter": ("melobot.utils.parse", "CmdArgFormatter", "3.1.1"),
        "CmdParser": ("melobot.utils.parse", "CmdParser", "3.1.1"),
        "CmdParserFactory": ("melobot.utils.parse", "CmdParserFactory", "3.1.1"),
        "FormatInfo": ("melobot.utils.parse", "CmdArgFormatInfo", "3.1.1"),
    },
)


def __getattr__(name: str) -> Any:
    return _LOADER.get(name)
